@@ -1,7 +1,7 @@
 /**
- * @file bloglines_source.c Bloglines feed list source support
+ * @file bloglines_source.c  Bloglines feed list source support
  * 
- * Copyright (C) 2006-2007 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2006-2009 Lars Lindner <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,15 +18,15 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <glib.h>
+#include "fl_sources/bloglines_source.h"
+
 #include <gtk/gtk.h>
 #include <unistd.h>
 
-#include "conf.h"
+#include "common.h"
+#include "feedlist.h"
 #include "subscription.h"
-#include "ui/ui_dialog.h"
-#include "fl_sources/bloglines_source.h"
-#include "fl_sources/node_source.h"
+#include "ui/liferea_dialog.h"
 #include "fl_sources/opml_source.h"
 
 static void bloglines_source_init (void) { }
@@ -40,7 +40,7 @@ on_bloglines_source_selected (GtkDialog *dialog,
                               gint response_id,
                               gpointer user_data) 
 {
-	nodePtr		node, parent = (nodePtr) user_data;
+	nodePtr		node;
 	subscriptionPtr	subscription;
 
 	if (response_id == GTK_RESPONSE_OK) {
@@ -48,11 +48,12 @@ on_bloglines_source_selected (GtkDialog *dialog,
 		subscription->updateOptions->username = g_strdup (gtk_entry_get_text (GTK_ENTRY (liferea_dialog_lookup (GTK_WIDGET(dialog), "userEntry"))));
 		subscription->updateOptions->password = g_strdup (gtk_entry_get_text (GTK_ENTRY (liferea_dialog_lookup (GTK_WIDGET(dialog), "passwordEntry"))));
 
-		node = node_new ();
+		node = node_new (node_source_get_node_type ());
 		node_set_title (node, "Bloglines");
 		node_source_new (node, bloglines_source_get_type ());
-		opml_source_setup (parent, node);
 		node_set_subscription (node, subscription);
+		feedlist_node_added (node);
+		
 		opml_source_update (node);
 	}
 
@@ -60,22 +61,15 @@ on_bloglines_source_selected (GtkDialog *dialog,
 }
 
 static void
-on_bloglines_source_dialog_destroy (GtkDialog *dialog,
-                                    gpointer user_data) 
-{
-	g_object_unref (user_data);
-}
-
-static void
-ui_bloglines_source_get_account_info (nodePtr parent)
+ui_bloglines_source_get_account_info (void)
 {
 	GtkWidget	*dialog;
 	
-	dialog = liferea_dialog_new ( PACKAGE_DATA_DIR G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S "bloglines_source.glade", "bloglines_source_dialog");
+	dialog = liferea_dialog_new ("bloglines_source.glade", "bloglines_source_dialog");
 	
 	g_signal_connect (G_OBJECT (dialog), "response",
 			  G_CALLBACK (on_bloglines_source_selected), 
-			  (gpointer) parent);
+			  NULL);
 }
 
 /* node source type definition */
