@@ -18,9 +18,11 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include "fl_sources/dummy_source.h"
+
 #include "common.h"
-#include "feedlist.h"
 #include "fl_sources/node_source.h"
+#include "ui/ui_common.h"
 
 static gchar * dummy_source_get_feedlist(nodePtr node) { return NULL; }
 
