@@ -1,7 +1,7 @@
 /**
- * @file node_source.h generic feed list provider interface
+ * @file node_source.h  generic node source interface
  * 
- * Copyright (C) 2005-2007 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2005-2009 Lars Lindner <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,11 +26,17 @@
 #include "node.h"
 #include "node_type.h"
 
-/* Liferea allows to have different sources in the feed list.
-   Sources can but do not need to be single instance only. Sources
+/* Liferea allows to have different sources in the feed list. These
+   sources are called "node sources" henceforth. Node sources can 
+   (but do not need to) be single instance only. Node sources do
    provide a subtree of the feed list that can be read-only
-   or not. A source might allow or not allow to add sub folders
-   and reorder (DnD) folder contents.
+   or not. A node source might allow or not allow to add sub folders
+   and reorder (DnD) folder contents. A node source might allow 
+   hierarchic grouping of its subtree or not. These properties
+   are determined by the node source type capability flags.
+   
+   The node source concept itself is a node type. The implementation
+   of this node type can be found in node_source.c.
 
    The default node source type must be capable of serving as the root
    node for all other source types. This mean it has to ensure to load
@@ -42,22 +48,23 @@
    A source type implementation can omit all callbacks marked as 
    optional. */
 
-#define NODE_SOURCE_TYPE_API_VERSION 5
+#define NODE_SOURCE_TYPE_API_VERSION 6
 
 enum {
 	NODE_SOURCE_CAPABILITY_IS_ROOT			= (1<<0),	/**< flag only for default feed list source */
 	NODE_SOURCE_CAPABILITY_MULTI_INSTANCES		= (1<<1),	/**< allows multiple source instances */
 	NODE_SOURCE_CAPABILITY_DYNAMIC_CREATION		= (1<<2),	/**< feed list source is user created */
-	NODE_SOURCE_CAPABILITY_WRITABLE_FEEDLIST	= (1<<3)	/**< the feed list tree of the source can be changed */
+	NODE_SOURCE_CAPABILITY_WRITABLE_FEEDLIST	= (1<<3),	/**< the feed list tree of the source can be changed */
+	NODE_SOURCE_CAPABILITY_HIERARCHIC_FEEDLIST	= (1<<4)	/**< the feed list tree of the source can have hierarchic folders */
 };
 
 /** feed list node source type */
 typedef struct nodeSourceType {
 	unsigned int	api_version;
 	
-	gchar		*id;		/**< a unique feed list source type identifier */
-	gchar		*name;		/**< a descriptive source name (for preferences and menus) */
-	gchar		*description;	/**< more detailed source type description (up to some sentences) */
+	const gchar	*id;		/**< a unique feed list source type identifier */
+	const gchar	*name;		/**< a descriptive source name (for preferences and menus) */
+	const gchar	*description;	/**< more detailed source type description (up to some sentences) */
 	gulong		capabilities;	/**< bitmask of feed list source capabilities */
 
 	/* source type loading and unloading methods */
@@ -70,7 +77,7 @@ typedef struct nodeSourceType {
 	 * the parent source node_request_add_*() implementation. 
 	 * Mandatory for all sources except the root source.
 	 */
-	void 		(*source_new)(nodePtr parent);
+	void 		(*source_new)(void);
 
 	/**
 	 * This OPTIONAL callback is used to delete an instance
@@ -120,6 +127,54 @@ typedef struct nodeSourceType {
 	 * during node_free() for a source node.
 	 */
 	void		(*free) (nodePtr node);
+
+	/**
+	 * Changes the flag state of an item.  This is to allow node source type 
+	 * implementations to synchronize remote item states.
+	 *
+	 * This is an OPTIONAL method.
+	 */
+	void		(*item_set_flag) (nodePtr node, itemPtr item, gboolean newState);
+
+	/**
+	 * Mark an item as read. This is to allow node source type 
+	 * implementations to synchronize remote item states.
+	 *
+	 * This is an OPTIONAL method.
+	 */
+	void            (*item_mark_read) (nodePtr node, itemPtr item, gboolean newState);
+	
+	/**
+	 * Add a new folder to the feed list provided by node
+	 * source. OPTIONAL, but must be implemented when
+	 * NODE_SOURCE_CAPABILITY_WRITABLE_FEEDLIST and
+	 * NODE_SOURCE_CAPABILITY_HIERARCHIC_FEEDLIST are set.
+	 */
+	nodePtr		(*add_folder) (nodePtr node, const gchar *title);
+
+	/**
+	 * Add a new subscription to the feed list provided
+	 * by the node source. OPTIONAL method, that must be implemented
+	 * when NODE_SOURCE_CAPABILITY_WRITABLE_FEEDLIST is set.
+	 *
+	 * The implementation could propagate the added subscription 
+	 * to a remote feed list service.
+	 *
+	 * The implementation MUST create and return a new child node 
+	 * setup with the given subscription which might be changed as necessary.
+	 *
+	 * The returned node will be automatically added to the feed list UI.
+	 * Initial update and state saving will be triggered automatically.
+	 */
+	nodePtr		(*add_subscription) (nodePtr node, struct subscription *subscription);
+	 
+	/**
+	 * Removes an existing node (subscription or folder) from the feed list 
+	 * provided by the node source. OPTIONAL method that must be
+	 * implemented when NODE_SOURCE_CAPABILITY_WRITABLE_FEEDLIST is set.
+	 */
+	void		(*remove_node) (nodePtr node, nodePtr child);
+
 } *nodeSourceTypePtr;
 
 /** feed list source instance */
@@ -141,15 +196,6 @@ typedef struct nodeSource {
  */
 nodePtr node_source_setup_root (void);
 
-/**
- * Registers a node source type.
- *
- * @param nodeSourceType	node source type
- *
- * @returns TRUE on success
- */
-gboolean node_source_type_register (nodeSourceTypePtr type);
-
 /**
  * Creates a new source and assigns it to the given new node. 
  * To be used to prepare a source node before adding it to the 
@@ -178,12 +224,50 @@ void node_source_update (nodePtr node);
 void node_source_auto_update (nodePtr node);
 
 /**
- * Launches a source creation dialog. The new source
- * instance will be added to the given node.
+ * Called when a new subscription has been added to the node source.
  *
- * @param node	the parent node
+ * @param node		the source node
+ * @param subscription	the new subscription
+ *
+ * @returns a new node intialized with the new subscription
  */
-void ui_node_source_type_dialog (nodePtr node);
+nodePtr node_source_add_subscription (nodePtr node, struct subscription *subscription);
+
+/**
+ * Called when an existing subscription is to be removed from a node source.
+ *
+ * @param node		the source node
+ * @param child		the child node to remove
+ */
+void node_source_remove_node (nodePtr node, nodePtr child);
+
+/**
+ * Called when a new folder is to be added to a node source feed list.
+ *
+ * @param node		the source node
+ * @param title		the folder title
+ *
+ * @returns a new node representing the new folder
+ */
+nodePtr node_source_add_folder (nodePtr node, const gchar *title);
+
+/**
+ * Called when the read state of an item changes.
+ *
+ * @param node		the source node 
+ * @param item		the affected item
+ * @param newState	the new item read state
+ */
+void node_source_item_mark_read (nodePtr node, itemPtr item, gboolean newState);
+
+/**
+ * Called when the flag state of an item changes.
+ *
+ * @param node		the source node 
+ * @param item		the affected item
+ * @param newState	the new item flag state
+ */
+void node_source_item_set_flag (nodePtr node, itemPtr item, gboolean newState);
 
 /* implementation of the node type interface */
 
