@@ -1,7 +1,7 @@
 /**
- * @file opml_source.c OPML Planet/Blogroll feed list source
+ * @file opml_source.c  OPML Planet/Blogroll feed list source
  * 
- * Copyright (C) 2006-2007 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2006-2009 Lars Lindner <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,12 +18,12 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <glib.h>
+#include "fl_sources/opml_source.h"
+
 #include <gtk/gtk.h>
 #include <unistd.h>
 
 #include "common.h"
-#include "db.h"
 #include "debug.h"
 #include "export.h"
 #include "feed.h"
@@ -31,84 +31,20 @@
 #include "folder.h"
 #include "node.h"
 #include "xml.h"
-#include "ui/ui_dialog.h"
+#include "ui/liferea_dialog.h"
+#include "ui/ui_common.h"
 #include "ui/ui_feedlist.h"
 #include "ui/ui_node.h"
-#include "fl_sources/opml_source.h"
-#include "fl_sources/node_source.h"
-#include "notification/notif_plugin.h"
 
 /** default OPML update interval = once a day */
 #define OPML_SOURCE_UPDATE_INTERVAL 60*60*24
 
-static void ui_opml_source_get_source_url (nodePtr parent);
-
-gchar * opml_source_get_feedlist(nodePtr node) {
-
-	return common_create_cache_filename("cache" G_DIR_SEPARATOR_S "plugins", node->id, "opml");
-}
-
-void opml_source_import(nodePtr node) {
-	gchar		*filename;
-
-	debug_enter("opml_source_import");
-
-	opml_source_setup(NULL, node);
-	
-	debug1(DEBUG_CACHE, "starting import of opml source instance (id=%s)", node->id);
-	filename = opml_source_get_feedlist(node);
-	if(g_file_test(filename, G_FILE_TEST_EXISTS)) {
-		import_OPML_feedlist(filename, node, node->source, FALSE, TRUE);
-	} else {
-		g_warning("cannot open \"%s\"", filename);
-		node->available = FALSE;
-	}
-	g_free(filename);
-	
-	subscription_set_update_interval (node->subscription, OPML_SOURCE_UPDATE_INTERVAL);
-
-	debug_exit("opml_source_import");
-}
-
-void opml_source_export(nodePtr node) {
-	gchar		*filename;
-	
-	debug_enter("opml_source_export");
-
-	/* Although the OPML structure won't change, it needs to
-	   be saved so that the feed ids are saved to disk after
-	   the first import or updates of the source OPML. */
-
-	g_assert(node == node->source->root);
-
-	filename = opml_source_get_feedlist(node);	   
-	export_OPML_feedlist(filename, node, TRUE);
-	g_free(filename);
-	
-	debug1(DEBUG_CACHE, "adding OPML source: title=%s", node_get_title(node));
-	
-	debug_exit("opml_source_export");
-}
-
-void opml_source_remove(nodePtr node) {
-	gchar		*filename;
-	
-	g_assert(node == node->source->root);
-
-	/* step 1: delete all feed cache files */
-	node_foreach_child(node, node_request_remove);
-	g_assert(!node->children);
-	
-	/* step 2: delete source instance OPML cache file */
-	filename = opml_source_get_feedlist(node);
-	unlink(filename);
-	g_free(filename);
-}
+/* OPML subscription list helper functions */
 
 typedef struct mergeCtxt {
-	nodePtr		rootNode;	/* root node of the OPML feed list source */
-	nodePtr		parent;		/* currently processed feed list node */
-	xmlNodePtr	xmlNode;	/* currently processed XML node of old OPML doc */
+	nodePtr		rootNode;	/**< root node of the OPML feed list source */
+	nodePtr		parent;		/**< currently processed feed list node */
+	xmlNodePtr	xmlNode;	/**< currently processed XML node of old OPML doc */
 } *mergeCtxtPtr;
 
 static void
@@ -135,17 +71,18 @@ opml_source_merge_feed (xmlNodePtr match, gpointer user_data)
 
 	if (!xpath_find (mergeCtxt->xmlNode, expr)) {
 		debug2(DEBUG_UPDATE, "adding %s (%s)", title, url);
-		node = node_new();
-		node_set_title (node, title);
 		if (url) {
-			node_set_type (node, feed_get_node_type ());
+			node = node_new (feed_get_node_type ());
 			node_set_data (node, feed_new ());
 			node_set_subscription (node, subscription_new (url, NULL, NULL));
 		} else {
-			node_set_type (node, folder_get_node_type ());
+			node = node_new (folder_get_node_type ());
 		}
-		node_add_child (mergeCtxt->parent, node, -1);
-		subscription_update (node->subscription, FEED_REQ_RESET_TITLE);
+		node_set_title (node, title);
+		node_set_parent (node, mergeCtxt->rootNode, -1);
+		feedlist_node_imported (node);
+		
+		subscription_update (node->subscription, FEED_REQ_RESET_TITLE | FEED_REQ_PRIORITY_HIGH);
 	}		
 	
 	/* Recursion if this is a folder */
@@ -196,18 +133,26 @@ opml_source_check_for_removal (nodePtr node, gpointer user_data)
 	
 	if (!xpath_find ((xmlNodePtr)user_data, expr)) {
 		debug1 (DEBUG_UPDATE, "removing %s...", node_get_title (node));
-		if (feedlist_get_selected () == node)
-			ui_feedlist_select (NULL);
-		node_request_remove (node);
+		feedlist_node_removed (node);
 	} else {
 		debug1 (DEBUG_UPDATE, "keeping %s...", node_get_title (node));
 	}
 	g_free (expr);
 }
 
-static void
-opml_source_process_update_result (nodePtr node, const struct updateResult * const result, updateFlags flags)
+/* OPML subscription type implementation */
+
+static gboolean
+opml_subscription_prepare_update_request (subscriptionPtr subscription, struct updateRequest *request)
 {
+	/* Nothing to do here for simple OPML subscriptions */
+	return TRUE;
+}
+
+static void
+opml_subscription_process_update_result (subscriptionPtr subscription, const struct updateResult * const result, updateFlags flags)
+{
+	nodePtr		node = subscription->node;
 	mergeCtxtPtr	mergeCtxt;
 	xmlDocPtr	doc, oldDoc;
 	xmlNodePtr	root, title;
@@ -271,16 +216,92 @@ opml_source_process_update_result (nodePtr node, const struct updateResult * con
 	node_foreach_child_data (node, node_update_subscription, GUINT_TO_POINTER (0));
 }
 
-static void
-opml_source_schedule_update (nodePtr node, updateFlags flags)
+/* subscription type definition */
+
+static struct subscriptionType opmlSubscriptionType = {
+	opml_subscription_prepare_update_request,
+	opml_subscription_process_update_result
+};
+
+/* OPML source type implementation */
+
+static void ui_opml_source_get_source_url (void);
+
+gchar *
+opml_source_get_feedlist (nodePtr node)
 {
-	subscription_update_with_callback (node->subscription, opml_source_process_update_result, flags);
+	return common_create_cache_filename ("cache" G_DIR_SEPARATOR_S "plugins", node->id, "opml");
+}
+
+void
+opml_source_import (nodePtr node)
+{
+	gchar	*filename;
+
+	debug_enter ("opml_source_import");
+
+	filename = g_strdup_printf ("%s.png", NODE_SOURCE_TYPE (node)->id);
+	node->icon = ui_common_create_pixbuf (filename);
+	g_free (filename);
+	
+	debug1 (DEBUG_CACHE, "starting import of opml source instance (id=%s)", node->id);
+	filename = opml_source_get_feedlist (node);
+	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
+		import_OPML_feedlist (filename, node, FALSE, TRUE);
+	} else {
+		g_warning ("cannot open \"%s\"", filename);
+		node->available = FALSE;
+	}
+	g_free (filename);
+	
+	subscription_set_update_interval (node->subscription, OPML_SOURCE_UPDATE_INTERVAL);
+	
+	node->subscription->type = &opmlSubscriptionType;
+
+	debug_exit ("opml_source_import");
+}
+
+void
+opml_source_export (nodePtr node)
+{
+	gchar		*filename;
+	
+	debug_enter ("opml_source_export");
+
+	/* Although the OPML structure won't change, it needs to
+	   be saved so that the feed ids are saved to disk after
+	   the first import or updates of the source OPML. */
+
+	g_assert (node == node->source->root);
+
+	filename = opml_source_get_feedlist (node);	   
+	export_OPML_feedlist (filename, node, TRUE);
+	g_free (filename);
+	
+	debug1 (DEBUG_CACHE, "adding OPML source: title=%s", node_get_title(node));
+	
+	debug_exit  ("opml_source_export");
+}
+
+void
+opml_source_remove (nodePtr node)
+{
+	gchar		*filename;
+	
+	/* step 1: delete all child nodes */
+	node_foreach_child (node, feedlist_node_removed);
+	g_assert (!node->children);
+	
+	/* step 2: delete source instance OPML cache file */
+	filename = opml_source_get_feedlist (node);
+	unlink (filename);
+	g_free (filename);
 }
 
 void
 opml_source_update (nodePtr node)
 {
-	opml_source_schedule_update (node, 0);  // FIXME: 0 ?
+	subscription_update (node->subscription, 0);  // FIXME: 0 ?
 }
 
 void
@@ -292,27 +313,7 @@ opml_source_auto_update (nodePtr node)
 	
 	/* do daily updates for the feed list and feed updates according to the default interval */
 	if (node->subscription->updateState->lastPoll.tv_sec + OPML_SOURCE_UPDATE_INTERVAL <= now.tv_sec)
-		opml_source_schedule_update (node, 0);
-}
-
-/** called during import and when subscribing, we will do
-    node_add_child() only when subscribing */
-void
-opml_source_setup (nodePtr parent, nodePtr node)
-{
-	gchar	*filename;
-	
-	node_set_type(node, node_source_get_node_type());
-	
-	filename = g_strdup_printf ("%s.png", NODE_SOURCE_TYPE (node)->id);
-	node->icon = create_pixbuf (filename);
-	g_free (filename);
-	
-	if(parent) {
-		gint pos;
-		ui_feedlist_get_target_folder(&pos);
-		node_add_child(parent, node, pos);
-	}
+		opml_source_update (node);
 }
 
 static void opml_source_init(void) { }
@@ -354,14 +355,20 @@ on_opml_source_selected (GtkDialog *dialog,
                          gint response_id,
                          gpointer user_data)
 {
-	nodePtr node, parent = (nodePtr)user_data;
+	nodePtr node;
+	subscriptionPtr	subscription;
+	const gchar *source;
 
 	if (response_id == GTK_RESPONSE_OK) {
-		node = node_new ();
+		node = node_new (node_source_get_node_type ());
 		node_set_title (node, OPML_SOURCE_DEFAULT_TITLE);
-		node_source_new (node, opml_source_get_type());
-		opml_source_setup (parent, node);
-		node_set_subscription (node, subscription_new (gtk_entry_get_text (GTK_ENTRY (liferea_dialog_lookup (GTK_WIDGET (dialog), "location_entry"))), NULL, NULL));
+		node_source_new (node, opml_source_get_type ());
+		source = gtk_entry_get_text (GTK_ENTRY (liferea_dialog_lookup (GTK_WIDGET (dialog), "location_entry")));
+		subscription = subscription_new (source, NULL, NULL);
+		subscription->type = &opmlSubscriptionType;
+		node_set_subscription (node, subscription);
+		feedlist_node_added (node);
+		
 		opml_source_update (node);
 	}
 
@@ -369,26 +376,7 @@ on_opml_source_selected (GtkDialog *dialog,
 }
 
 static void
-on_opml_source_dialog_destroy (GtkDialog *dialog,
-                               gpointer user_data) 
-{
-	g_object_unref (user_data);
-}
-
-static void
-ui_opml_source_get_source_url (nodePtr parent) 
-{
-	GtkWidget	*dialog;
-
-	dialog = liferea_dialog_new (PACKAGE_DATA_DIR G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S "opml_source.glade", "opml_source_dialog");
-
-	g_signal_connect (G_OBJECT (dialog), "response",
-			  G_CALLBACK (on_opml_source_selected), 
-			  (gpointer)parent);
-}
-
-static void
-on_file_select_clicked (const gchar *filename, gpointer user_data)
+on_opml_file_selected (const gchar *filename, gpointer user_data)
 {
 	GtkWidget	*dialog = GTK_WIDGET (user_data);
 
@@ -396,8 +384,25 @@ on_file_select_clicked (const gchar *filename, gpointer user_data)
 		gtk_entry_set_text (GTK_ENTRY (liferea_dialog_lookup (dialog, "location_entry")), g_strdup(filename));
 }
 
-void
-on_select_button_clicked (GtkButton *button, gpointer user_data)
+static void
+on_opml_file_choose_clicked (GtkButton *button, gpointer user_data)
 {
-	ui_choose_file (_("Choose OPML File"), GTK_STOCK_OPEN, FALSE, on_file_select_clicked, NULL, NULL, user_data);
+	ui_choose_file (_("Choose OPML File"), GTK_STOCK_OPEN, FALSE, on_opml_file_selected, NULL, NULL, user_data);
+}
+
+static void
+ui_opml_source_get_source_url (void) 
+{
+	GtkWidget	*dialog, *button;
+
+	dialog = liferea_dialog_new ("opml_source.glade", "opml_source_dialog");
+	button = liferea_dialog_lookup (dialog, "select_button");
+
+	g_signal_connect (G_OBJECT (dialog), "response",
+			  G_CALLBACK (on_opml_source_selected), 
+			  NULL);
+			  
+	g_signal_connect (G_OBJECT (button), "clicked",
+	                  G_CALLBACK (on_opml_file_choose_clicked),
+			  dialog);
 }
