@@ -1,7 +1,7 @@
 /**
- * @file opml_source.h OPML Planet/Blogroll feed list provider
+ * @file opml_source.h  OPML Planet/Blogroll feed list provider
  * 
- * Copyright (C) 2005-2007 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2005-2009 Lars Lindner <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,8 +21,8 @@
 #ifndef _OPML_SOURCE_H
 #define _OPML_SOURCE_H
 
-#include "common.h"
 #include "node.h"
+#include "fl_sources/node_source.h"
 
 #define OPML_SOURCE_DEFAULT_TITLE _("New OPML Subscription")
 
@@ -58,14 +58,6 @@ void opml_source_export(nodePtr node);
  */
 void opml_source_remove(nodePtr node);
 
-/**
- * Sets up the given node as a OPML source node.
- *
- * @param parent	parent node (or NULL when importing)
- * @param node		the node
- */
-void opml_source_setup(nodePtr parent, nodePtr node);
-
 /**
  * Force update of the OPML source and all child subscriptions of the given node.
  *
