@@ -1,7 +1,7 @@
 /**
- * @file itemlist.c itemlist handling
+ * @file itemlist.c  itemlist handling
  *
- * Copyright (C) 2004-2007 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2004-2008 Lars Lindner <lars.lindner@gmail.com>
  *	      
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,12 +18,8 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
-
 #include <string.h>
- 
+
 #include "comments.h"
 #include "common.h"
 #include "conf.h"
@@ -32,17 +28,19 @@
 #include "feed.h"
 #include "feedlist.h"
 #include "folder.h"
+#include "item_state.h"
 #include "itemlist.h"
 #include "itemset.h"
-#include "itemview.h"
+#include "metadata.h"
 #include "node.h"
 #include "rule.h"
 #include "script.h"
 #include "vfolder.h"
+#include "ui/itemview.h"
+#include "ui/liferea_shell.h"
 #include "ui/ui_feedlist.h"
 #include "ui/ui_itemlist.h"
-#include "ui/ui_htmlview.h"
-#include "ui/ui_mainwindow.h"
+#include "ui/liferea_htmlview.h"
 #include "ui/ui_node.h"
 
 /* This is a simple controller implementation for itemlist handling. 
@@ -59,12 +57,16 @@ static struct itemlist_priv
 	gulong		selectedId;		/**< the currently selected (and displayed) item id */
 	gboolean	hasEnclosures;		/**< TRUE if at least one item of the current itemset has an enclosure */
 	
-	guint 		viewMode;		/**< current viewing mode */
+	nodeViewType	viewMode;		/**< current viewing mode */
 	guint 		loading;		/**< if >0 prevents selection effects when loading the item list */
 	itemPtr		invalidSelection;	/**< if set then the next selection might need to do an unselect first */
 
 	gboolean 	deferredRemove;		/**< TRUE if selected item needs to be removed from cache on unselecting */
 	gboolean 	deferredFilter;		/**< TRUE if selected item needs to be filtered on unselecting */
+	
+	gboolean	isSearchResult;		/**< TRUE if a search result is displayed */
+	gboolean	searchResultComplete;	/**< TRUE if search result merging is complete */
+	
 } itemlist_priv;
 
 static void
@@ -132,6 +134,12 @@ itemlist_get_selected (void)
 	return item_load(itemlist_priv.selectedId);
 }
 
+gulong
+itemlist_get_selected_id (void)
+{
+	return itemlist_priv.selectedId;
+}
+
 static void
 itemlist_set_selected (itemPtr item)
 {
@@ -182,13 +190,14 @@ static gboolean
 itemlist_filter_check_item (itemPtr item)
 {
 	/* use search folder rule list in case of a search folder */
-	if ((itemlist_priv.currentNode != NULL) &&
-	    IS_VFOLDER (itemlist_priv.currentNode))
-		return rules_check_item (((vfolderPtr)itemlist_priv.currentNode->data)->rules, item);
+	if (itemlist_priv.currentNode && IS_VFOLDER (itemlist_priv.currentNode)) {
+		vfolderPtr vfolder = (vfolderPtr)itemlist_priv.currentNode->data;
+		return rules_check_item (vfolder->rules, vfolder->anyMatch, item);
+	}
 
 	/* apply the item list filter if available */
 	if (itemlist_priv.filter)
-		return rules_check_item (itemlist_priv.filter, item);
+		return rules_check_item (itemlist_priv.filter, TRUE /* OR logic */, item);
 	
 	/* otherwise keep the item */
 	return TRUE;
@@ -216,27 +225,36 @@ itemlist_merge_item (itemPtr item)
 void
 itemlist_merge_itemset (itemSetPtr itemSet) 
 {
-	nodePtr		node;
-
 	debug_enter ("itemlist_merge_itemset");
 	
 	debug_start_measurement (DEBUG_GUI);
 	
-	node = node_from_id(itemSet->nodeId);
+	/* No node check when loading search results directly */
+	if (!itemlist_priv.isSearchResult) {
+		nodePtr node = node_from_id (itemSet->nodeId);
 
-	if(!itemlist_priv.currentNode)
-		return; /* Nothing to do if nothing is displayed */
-	
-	if(!IS_VFOLDER (itemlist_priv.currentNode) &&
-	   (itemlist_priv.currentNode != node) && 
-	   !node_is_ancestor (itemlist_priv.currentNode, node))
-		return; /* Nothing to do if the item set does not belong to this node, or this is a search folder */
-
-	if(IS_FOLDER (itemlist_priv.currentNode) && 
-	   (0 == conf_get_int_value (FOLDER_DISPLAY_MODE)))
-		return; /* Bail out if it is a folder without the recursive display preference set */
+		if (!itemlist_priv.currentNode)
+			return; /* Nothing to do if nothing is displayed */
 		
-	debug1 (DEBUG_GUI, "reloading item list with node \"%s\"", node_get_title (node));
+		if (!IS_VFOLDER (itemlist_priv.currentNode) &&
+		    (itemlist_priv.currentNode != node) && 
+		    !node_is_ancestor (itemlist_priv.currentNode, node))
+			return; /* Nothing to do if the item set does not belong to this node, or this is a search folder */
+
+		if (IS_FOLDER (itemlist_priv.currentNode) && 
+		    (0 == conf_get_int_value (FOLDER_DISPLAY_MODE)))
+			return; /* Bail out if it is a folder without the recursive display preference set */
+			
+		debug1 (DEBUG_GUI, "reloading item list with node \"%s\"", node_get_title (node));
+	} else {
+		/* If we are loading a search result we must never merge 
+		   anything besides the search items. In fact if we already
+		   have items we just return. */
+		if (itemlist_priv.searchResultComplete)
+			return;
+			
+		itemlist_priv.searchResultComplete = TRUE;
+	}
 
 	/* merge items into item view */
 	itemset_foreach (itemSet, itemlist_merge_item);
@@ -251,15 +269,25 @@ itemlist_merge_itemset (itemSetPtr itemSet)
 	debug_exit ("itemlist_merge_itemset");
 }
 
+void
+itemlist_load_search_result (itemSetPtr itemSet)
+{
+	itemview_set_mode (ITEMVIEW_SINGLE_ITEM);
+	
+	itemlist_priv.isSearchResult = TRUE;
+	itemlist_priv.searchResultComplete = FALSE;	/* enable result merging */
+	itemlist_merge_itemset (itemSet);	
+	itemlist_priv.searchResultComplete = TRUE;	/* disable result merging */
+}
+
 /** 
- * To be called whenever a feed was selected and should
+ * To be called whenever a node was selected and should
  * replace the current itemlist.
  */
 void
 itemlist_load (nodePtr node) 
 {
 	itemSetPtr	itemSet;
-	GSList		*iter;
 
 	debug_enter ("itemlist_load");
 
@@ -270,6 +298,7 @@ itemlist_load (nodePtr node)
 	g_assert (!itemlist_priv.guids);
 	g_assert (!itemlist_priv.filter);
 	itemlist_priv.hasEnclosures = FALSE;
+	itemlist_priv.isSearchResult = FALSE;
 
 	/* 1. Filter check. Don't continue if folder is selected and 
 	   no folder viewing is configured. If folder viewing is enabled
@@ -277,30 +306,33 @@ itemlist_load (nodePtr node)
 
 	/* for folders and other heirarchic nodes do filtering */
 	if (IS_FOLDER (node) || node->children) {
+		liferea_shell_update_allitems_actions (FALSE, 0 != node->unreadCount);
+		
 		if (0 == conf_get_int_value (FOLDER_DISPLAY_MODE))
 			return;
 	
 		if (conf_get_bool_value (FOLDER_DISPLAY_HIDE_READ))
 			itemlist_priv.filter = g_slist_append (NULL, rule_new (NULL, "unread", "", TRUE));
+	} else {
+		liferea_shell_update_allitems_actions (0 != node->itemCount, 0 != node->unreadCount);
 	}
 
 	itemlist_priv.loading++;
 	itemlist_priv.viewMode = node_get_view_mode (node);
-	ui_mainwindow_set_layout (itemlist_priv.viewMode);
-	ui_mainwindow_update_item_menu (FALSE);
+	itemview_set_layout (itemlist_priv.viewMode);
 
 	/* Set the new displayed node... */
 	itemlist_priv.currentNode = node;
 	itemview_set_displayed_node (itemlist_priv.currentNode);
 
-	if(NODE_VIEW_MODE_COMBINED != node_get_view_mode (node))
+	if (NODE_VIEW_MODE_COMBINED != node_get_view_mode (node))
 		itemview_set_mode (ITEMVIEW_NODE_INFO);
 	else
 		itemview_set_mode (ITEMVIEW_ALL_ITEMS);
 	
-	itemSet = node_get_itemset(itemlist_priv.currentNode);
-	itemlist_merge_itemset(itemSet);
-	itemset_free(itemSet);
+	itemSet = node_get_itemset (itemlist_priv.currentNode);
+	itemlist_merge_itemset (itemSet);
+	itemset_free (itemSet);
 
 	itemlist_priv.loading--;
 
@@ -389,7 +421,7 @@ itemlist_select_next_unread (void)
 				result = itemlist_find_unread_item ();	/* find first unread item */
 		} else {
 			/* if we don't find a feed with unread items do nothing */
-			ui_mainwindow_set_status_bar (_("There are no unread items "));
+			liferea_shell_set_status_bar (_("There are no unread items "));
 		}
 	}
 
@@ -404,14 +436,14 @@ itemlist_select_next_unread (void)
 void
 itemlist_toggle_flag (itemPtr item) 
 {
-	item_state_set_flagged (item, !(item->flagStatus));
+	item_set_flag_state (item, !(item->flagStatus));
 	itemview_update ();
 }
 
 void
 itemlist_toggle_read_status (itemPtr item) 
 {
-	item_state_set_read (item, !(item->readStatus));
+	item_set_read_state (item, !(item->readStatus));
 	itemview_update ();
 }
 
@@ -555,33 +587,30 @@ itemlist_selection_changed (itemPtr item)
 	
 		/* set read and unset update status when selecting */
 		if (item) {
-			comments_refresh (item);
+			nodePtr node = node_from_id (item->nodeId);
+			
+			if (IS_FEED(node) && !((feedPtr)node->data)->ignoreComments)
+				comments_refresh (item);
 
-			item_state_set_read (item, TRUE);
-			item_state_set_updated (item, FALSE);
+			item_set_read_state (item, TRUE);
 
-			if(node_load_link_preferred (node_from_id (item->nodeId))) {
-				liferea_htmlview_launch_URL (ui_mainwindow_get_active_htmlview (), 
-				                             item_get_source (itemlist_get_selected ()), UI_HTMLVIEW_LAUNCH_INTERNAL);
+			if (node_load_link_preferred (node)) {
+				itemview_launch_URL (item_get_source (item), TRUE /* force internal */);
 			} else {
 				itemview_set_mode (ITEMVIEW_SINGLE_ITEM);
 				itemview_select_item (item);
 				itemview_update ();
 			}
 			ui_node_update (item->nodeId);
+			
 		}
-		
-		ui_mainwindow_update_item_menu (NULL != item);
 
 		feedlist_reset_new_item_count ();
-	} else {
-		/* If this happens we got an unwanted item selection
-		   caused by some automatic feed selection. We need
-		   to unselect before making the next selection, 
-		   otherwise it could fail */
-		itemview_set_invalid_selection (item);
 	}
 
+	if (item)
+		item_unload (item);
+	
 	debug_end_measurement (DEBUG_GUI, "itemlist selection");
 	debug_exit ("itemlist_selection_changed");
 }
@@ -606,7 +635,7 @@ itemlist_set_view_mode (guint newMode)
 		itemlist_unload (FALSE);
 		
 		node_set_view_mode (node, itemlist_priv.viewMode);
-		ui_mainwindow_set_layout (itemlist_priv.viewMode);
+		itemview_set_layout (itemlist_priv.viewMode);
 		itemlist_load (node);
 	}
 }
