@@ -1,7 +1,7 @@
 /**
- * @file node.c common feed list node handling
+ * @file node.c  hierarchic feed list node handling
  * 
- * Copyright (C) 2003-2007 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2009 Lars Lindner <lars.lindner@gmail.com>
  * Copyright (C) 2004-2006 Nathan J. Conrad <t98502@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,19 +26,21 @@
 #include "db.h"
 #include "debug.h"
 #include "favicon.h"
-#include "feed.h"
+#include "feed.h"	// FIXME
 #include "feedlist.h"
 #include "folder.h"
 #include "itemset.h"
+#include "item_state.h"
 #include "node.h"
+#include "node_view.h"
 #include "render.h"
 #include "script.h"
 #include "update.h"
 #include "vfolder.h"
 #include "fl_sources/node_source.h"
-#include "ui/ui_feedlist.h"
-#include "ui/ui_itemlist.h"
+#include "ui/liferea_shell.h"
 #include "ui/ui_node.h"
+#include "ui/ui_tray.h"
 
 static GHashTable *nodes = NULL;
 
@@ -77,12 +79,15 @@ nodePtr node_from_id(const gchar *id) {
 	return node;
 }
 
-nodePtr node_new(void) {
+nodePtr node_new(nodeTypePtr type) {
 	nodePtr	node;
 	gchar	*id;
+	
+	g_assert (NULL != type);
 
 	node = (nodePtr)g_new0(struct node, 1);
-	node->sortColumn = IS_TIME;
+	node->type = type;
+	node->sortColumn = NODE_VIEW_SORT_BY_TIME;
 	node->sortReversed = TRUE;	/* default sorting is newest date at top */
 	node->available = TRUE;
 	node_set_icon(node, NULL);	/* initialize favicon file name */
@@ -204,24 +209,26 @@ node_calc_counters (nodePtr node)
 	NODE_TYPE (node)->update_counters (node);
 }
 
-static void node_update_parent_counters(nodePtr node) {
+static void
+node_update_parent_counters (nodePtr node)
+{
 	guint old;
 
-	if(!node)
+	if (!node)
 		return;
 		
 	old = node->unreadCount;
 
-	NODE_TYPE(node)->update_counters(node);
+	NODE_TYPE (node)->update_counters (node);
 	
 	if (old != node->unreadCount) {
 		ui_node_update (node->id);
 		ui_tray_update ();
-		ui_mainwindow_update_feedsinfo ();
+		liferea_shell_update_unread_stats ();
 	}
 	
-	if(node->parent)
-		node_update_parent_counters(node->parent);
+	if (node->parent)
+		node_update_parent_counters (node->parent);
 }
 
 void node_update_counters(nodePtr node) {
@@ -230,7 +237,7 @@ void node_update_counters(nodePtr node) {
 	/* Update the node itself and its children */
 	node_calc_counters(node);
 	
-	if (old != node->unreadCount);
+	if (old != node->unreadCount)
 		ui_node_update (node->id);
 		
 	/* Update the unread count of the parent nodes,
@@ -240,37 +247,22 @@ void node_update_counters(nodePtr node) {
 
 // FIXME: bad interface, do not imply node icons are favicons
 void
-node_update_favicon (nodePtr node, GTimeVal *now)
+node_update_favicon (nodePtr node)
 {
 	if (IS_FEED (node)) {
 		debug1 (DEBUG_UPDATE, "favicon of node %s needs to be updated...", node->title);
-		subscription_update_favicon (node->subscription, now);
+		subscription_update_favicon (node->subscription);
 	}
 	
 	/* Recursion */
 	if (node->children)
-		node_foreach_child_data (node, node_update_favicon, now);
+		node_foreach_child (node, node_update_favicon);
 }
 
 itemSetPtr
 node_get_itemset (nodePtr node)
 {
-	itemSetPtr itemSet;
-	
-	debug_enter ("node_get_itemset");
-	
-	itemSet = NODE_TYPE (node)->load (node);
-	
-	debug_exit ("node_get_itemset");
-	
-	return itemSet;
-}
-
-void
-node_process_update_result (nodePtr node, const struct updateResult * const result, updateFlags flags)
-{
-	if (NODE_TYPE (node)->process_update_result)
-		NODE_TYPE (node)->process_update_result (node, result, flags);
+	return NODE_TYPE (node)->load (node);
 }
 
 void
@@ -278,9 +270,9 @@ node_mark_all_read (nodePtr node)
 {
 	if (!node)
 		return;
-		
+
 	if (0 != node->unreadCount) {
-		item_state_set_all_read (node);
+		itemset_mark_read (node);
 		node->unreadCount = 0;
 		node->needsUpdate = TRUE;
 	}
@@ -297,77 +289,29 @@ node_render(nodePtr node)
 
 /* import callbacks and helper functions */
 
-void node_add_child(nodePtr parent, nodePtr node, gint position) {
+void
+node_set_parent (nodePtr node, nodePtr parent, gint position)
+{
+	g_assert (NULL != parent);
 
-	if(!parent)
-		parent = ui_feedlist_get_target_folder(&position);	
-
-	parent->children = g_slist_insert(parent->children, node, position);
+	parent->children = g_slist_insert (parent->children, node, position);
 	node->parent = parent;
 	
-	/* new node may be provided by another feed list handler, if 
-	   not they are handled by the parents handler */
-	if(!node->source)
+	/* new nodes may be provided by another node source, if 
+	   not they are handled by the parents node source */
+	if (!node->source)
 		node->source = parent->source;
-	
-	ui_node_add(parent, node, position);	
-}
-
-/* To be called by node type implementations to add nodes */
-void node_add(nodePtr node, nodePtr parent, gint pos, guint flags) {
-
-	debug1(DEBUG_GUI, "new node will be added to folder \"%s\"", node_get_title(parent));
-
-	ui_feedlist_get_target_folder(&pos);
-
-	node_add_child(parent, node, pos);
-	subscription_update(node->subscription, flags);
-}
-
-/* Automatic subscription adding (e.g. URL DnD), creates a new feed node
-   without any user interaction. */
-void
-node_request_automatic_add (const gchar *source, const gchar *title, const gchar *filter, updateOptionsPtr options, gint flags)
-{
-	nodePtr		node, parent;
-	gint		pos;
-
-	g_assert(NULL != source);
-
-	parent = feedlist_get_insertion_point();
-
-	if(0 == (NODE_SOURCE_TYPE(parent->source->root)->capabilities & NODE_CAPABILITY_ADD_CHILDS))
-		return;
-
-	node = node_new ();
-	node_set_type (node,feed_get_node_type ());
-	node_set_title (node, title?title:_("New Subscription"));
-	node_set_data (node, feed_new ());
-	node_set_subscription (node, subscription_new (source, filter, options));
-	db_subscription_update (node->subscription);
-
-	ui_feedlist_get_target_folder (&pos);
-	node_add_child (parent, node, pos);
-	subscription_update (node->subscription, flags);
-	feedlist_schedule_save ();
-	ui_feedlist_select (node);
-	
-	script_run_for_hook (SCRIPT_HOOK_NEW_SUBSCRIPTION);
 }
 
 void
-node_request_remove (nodePtr node)
+node_remove (nodePtr node)
 {
-	/* using itemlist_remove_all_items() ensure correct unread
+	/* using itemlist_remove_all_items() ensures correct unread
 	   and item counters for all parent folders and matching 
 	   search folders */
 	itemlist_remove_all_items (node);
 	
 	NODE_TYPE (node)->remove (node);
-
-	node->parent->children = g_slist_remove (node->parent->children, node);
-
-	node_free (node);
 }
 
 static xmlDocPtr node_to_xml(nodePtr node) {
@@ -423,18 +367,19 @@ node_set_title (nodePtr node, const gchar *title)
 
 const gchar * node_get_title(nodePtr node) { return node->title; }
 
-void node_set_icon(nodePtr node, gpointer icon) {
-
-	if(node->icon) 
-		g_object_unref(node->icon);
+void
+node_set_icon (nodePtr node, gpointer icon)
+{
+	if (node->icon) 
+		g_object_unref (node->icon);
 	node->icon = icon;
 	
-	if(node->iconFile)
-		g_free(node->iconFile);
-	if(node->icon)
-		node->iconFile = common_create_cache_filename("cache" G_DIR_SEPARATOR_S "favicons", node->id, "png");
+	g_free (node->iconFile);
+	
+	if (node->icon)
+		node->iconFile = common_create_cache_filename ("cache" G_DIR_SEPARATOR_S "favicons", node->id, "png");
 	else
-		node->iconFile = g_strdup(PACKAGE_DATA_DIR G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S "pixmaps" G_DIR_SEPARATOR_S "default.png");
+		node->iconFile = g_build_filename (PACKAGE_DATA_DIR, PACKAGE, "pixmaps", "default.png", NULL);
 }
 
 /** determines the nodes favicon or default icon */
@@ -446,9 +391,6 @@ gpointer node_get_icon(nodePtr node) {
 	if(!icon)
 		icon = NODE_TYPE(node)->icon;
 
-	if(!node->available)
-		icon = icons[ICON_UNAVAILABLE];
-
 	return icon;
 }
 
@@ -470,16 +412,30 @@ void node_set_id(nodePtr node, const gchar *id) {
 
 const gchar *node_get_id(nodePtr node) { return node->id; }
 
-void node_set_sort_column(nodePtr node, gint sortColumn, gboolean reversed) {
+gboolean
+node_set_sort_column (nodePtr node, gint sortColumn, gboolean reversed)
+{
+	if (node->sortColumn == sortColumn &&
+	    node->sortReversed == reversed)
+	    	return FALSE;
 
 	node->sortColumn = sortColumn;
 	node->sortReversed = reversed;
-	feedlist_schedule_save();
+
+	return TRUE;
 }
 
-void node_set_view_mode(nodePtr node, guint newMode) { node->viewMode = newMode; }
+void
+node_set_view_mode (nodePtr node, nodeViewType viewMode)
+{
+	node->viewMode = viewMode;
+}
 
-gboolean node_get_view_mode(nodePtr node) { return node->viewMode; }
+nodeViewType
+node_get_view_mode (nodePtr node)
+{
+	return node->viewMode;
+}
 
 gboolean
 node_load_link_preferred (nodePtr node)
@@ -495,8 +451,8 @@ node_get_base_url(nodePtr node)
 {
 	const gchar 	*baseUrl = NULL;
 
-	if (IS_FEED (node))
-		baseUrl = feed_get_html_url ((feedPtr)node->data);
+	if (node->subscription)
+		baseUrl = subscription_get_homepage (node->subscription);
 
 	/* prevent feed scraping commands to end up as base URI */
 	if (!((baseUrl != NULL) &&
