@@ -1,7 +1,7 @@
 /**
- * @file pie_entry.c Atom0.3 entry parsing 
+ * @file pie_entry.c  Atom0.3 entry parsing 
  *
- * Copyright (C) 2003-2007 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2008 Lars Lindner <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,28 +18,18 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
-
 #include <string.h>
 
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
 
 #include "common.h"
+#include "feed_parser.h"
 #include "pie_entry.h"
 #include "pie_feed.h"
 #include "metadata.h"
 #include "xml.h"
 
-/* uses the same namespace handler as PIE_channel */
-extern GHashTable *pie_nstable;
-extern GHashTable *ns_pie_ns_uri_table;
-
-/* we reuse some pie_feed.c function */
-extern gchar * parseAuthor(xmlNodePtr cur);
-
 /* <content> tag support, FIXME: base64 not supported */
 /* method to parse standard tags for each item element */
 itemPtr parseEntry(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
@@ -66,7 +56,7 @@ itemPtr parseEntry(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 			if((cur->ns->href && (nsh = (NsHandler *)g_hash_table_lookup(ns_pie_ns_uri_table, (gpointer)cur->ns->href))) ||
 			   (cur->ns->prefix && (nsh = (NsHandler *)g_hash_table_lookup(pie_nstable, (gpointer)cur->ns->prefix)))) {
 				
-				if(pf = nsh->parseItemTag)
+				if(NULL != (pf = nsh->parseItemTag))
 					(*pf)(ctxt, cur);
 				cur = cur->next;
 				continue;
@@ -76,23 +66,23 @@ itemPtr parseEntry(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 		} /* explicitly no following else !!! */
 		
 		if(!xmlStrcmp(cur->name, BAD_CAST"title")) {
-			if(tmp = unhtmlize(pie_parse_content_construct(cur))) {
+			if(NULL != (tmp = unhtmlize(pie_parse_content_construct(cur)))) {
 				item_set_title(ctxt->item, tmp);
 				g_free(tmp);
 			}
 		} else if(!xmlStrcmp(cur->name, BAD_CAST"link")) {
-			if(tmp2 = common_utf8_fix(xmlGetProp(cur, BAD_CAST"href"))) {
+			if(NULL != (tmp2 = xml_get_attribute(cur, "href"))) {
 				/* 0.3 link : rel, type and href attribute */
 				xtmp = xmlGetProp(cur, BAD_CAST"rel");
 				if(xtmp != NULL && !xmlStrcmp(xtmp, BAD_CAST"alternate"))
 					item_set_source(ctxt->item, tmp2);
-				else
-					/* FIXME: Maybe do something with other links? */;
+				/* else
+					FIXME: Maybe do something with other links? */
 				xmlFree(xtmp);
 				g_free(tmp2);
 			} else {
 				/* 0.2 link : element content is the link, or non-alternate link in 0.3 */
-				if(tmp = common_utf8_fix(xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1))) {
+				if(NULL != (tmp = common_utf8_fix((gchar *)xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1)))) {
 					item_set_source(ctxt->item, tmp);
 					g_free(tmp);
 				}
@@ -108,19 +98,19 @@ itemPtr parseEntry(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 			ctxt->item->metadata = metadata_list_append(ctxt->item->metadata, "contributor", tmp);
 			g_free(tmp);
 		} else if(!xmlStrcmp(cur->name, BAD_CAST"id")) {
-			if(tmp = common_utf8_fix(xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1))) {
+			if(NULL != (tmp = common_utf8_fix((gchar *)xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1)))) {
 				item_set_id(ctxt->item, tmp);
 				g_free(tmp);
 			}
 		} else if(!xmlStrcmp(cur->name, BAD_CAST"issued")) {
 			/* FIXME: is <modified> or <issued> or <created> the time tag we want to display? */
- 			if(tmp = common_utf8_fix(xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1))) {
+ 			if(NULL != (tmp = common_utf8_fix((gchar *)xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1)))) {
 				ctxt->item->time = parseISO8601Date(tmp);
 				g_free(tmp);
 			}
 		} else if(!xmlStrcmp(cur->name, BAD_CAST"content")) {
 			/* <content> support */
-			if(tmp = common_utf8_fix(pie_parse_content_construct(cur))) {
+			if(NULL != (tmp = common_utf8_fix(pie_parse_content_construct(cur)))) {
 				item_set_description(ctxt->item, tmp);
 				g_free(tmp);
 			}
@@ -128,13 +118,13 @@ itemPtr parseEntry(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 			/* <summary> can be used for short text descriptions, if there is no
 			   <content> description we show the <summary> content */
 			if(!item_get_description(ctxt->item)) {
-				if(tmp = common_utf8_fix(pie_parse_content_construct(cur))) {
+				if(NULL != (tmp = common_utf8_fix(pie_parse_content_construct(cur)))) {
 					item_set_description(ctxt->item, tmp);
 					g_free(tmp);
 				}
 			}
 		} else if(!xmlStrcmp(cur->name, BAD_CAST"copyright")) {
- 			if(tmp = common_utf8_fix(xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1))) {
+ 			if(NULL != (tmp = common_utf8_fix((gchar *)xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1)))) {
 				ctxt->item->metadata = metadata_list_append(ctxt->item->metadata, "copyright", tmp);
 				g_free(tmp);
 			}
