@@ -1,7 +1,7 @@
 /**
  * @file pie_feed.c Atom 0.3 channel parsing
  * 
- * Copyright (C) 2003-2007 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2008 Lars Lindner <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
 #include "common.h"
-#include "feed.h"
+#include "feed_parser.h"
 #include "feedlist.h"
 #include "pie_feed.h"
 #include "pie_entry.h"
@@ -49,19 +49,21 @@ GHashTable	*ns_pie_ns_uri_table = NULL;
 	"created",  <---- Not in the specs for feeds
 */
 gchar* pie_parse_content_construct(xmlNodePtr cur) {
-	gchar	*mode, *type, *tmp, *ret;
+	gchar	*mode, *type, *ret;
 
 	g_assert(NULL != cur);
 	ret = NULL;
 	
 	/* determine encoding mode */
-	mode = common_utf8_fix(xmlGetProp(cur, BAD_CAST"mode"));
-	type = common_utf8_fix(xmlGetProp(cur, BAD_CAST"type"));
+	mode = xml_get_attribute (cur, "mode");
+	type = xml_get_attribute (cur, "type");
 
 	/* Modes are used in older versions of ATOM, including 0.3. It
 	   does not exist in the newer IETF drafts.*/
 	if(NULL != mode) {
 		if(!strcmp(mode, "escaped")) {
+			gchar	*tmp;
+
 			tmp = common_utf8_fix(xhtml_extract (cur, 0, NULL));
 			if(NULL != tmp)
 				ret = tmp;
@@ -84,18 +86,18 @@ gchar* pie_parse_content_construct(xmlNodePtr cur) {
 
 		/* This that need to be de-encoded and should not contain sub-tags.*/
 		if(NULL == type ||
-			!g_strcasecmp(type, "TEXT") ||
+			!g_ascii_strcasecmp(type, "TEXT") ||
 			!strcmp(type, "text/plain")) {
 			gchar *tmp;
-			tmp = common_utf8_fix(xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
+			tmp = common_utf8_fix((gchar *)xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
 			ret = g_markup_printf_escaped("<div xmlns=\"http://www.w3.org/1999/xhtml\"><pre>%s</pre></div>", tmp);
 			g_free(tmp);
 			/* Next are things that contain subttags */
-		} else if(!g_strcasecmp(type, "HTML") ||
+		} else if(!g_ascii_strcasecmp(type, "HTML") ||
 		          !strcmp(type, "text/html")) {
 			ret = xhtml_extract (cur, 0,"http://default.base.com/");
 		} else if(/* HTML types */
-		          !g_strcasecmp(type, "xhtml") ||
+		          !g_ascii_strcasecmp(type, "xhtml") ||
 		          !strcmp(type, "application/xhtml+xml")) {
 			ret = xhtml_extract (cur, 1,"http://default.base.com/");
 		}
@@ -111,7 +113,6 @@ gchar* pie_parse_content_construct(xmlNodePtr cur) {
 }
 
 
-/* nonstatic because used by pie_entry.c too */
 gchar * parseAuthor(xmlNodePtr cur) {
 	gchar	*tmp = NULL;
 	gchar	*tmp2, *tmp3;
@@ -126,10 +127,10 @@ gchar * parseAuthor(xmlNodePtr cur) {
 		}
 		
 		if (!xmlStrcmp(cur->name, BAD_CAST"name"))
-			tmp = common_utf8_fix(xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
+			tmp = common_utf8_fix((gchar *)xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
 
 		if (!xmlStrcmp(cur->name, BAD_CAST"email")) {
-			tmp2 = common_utf8_fix(xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
+			tmp2 = common_utf8_fix((gchar *)xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
 			tmp3 = g_strdup_printf("%s <a href=\"mailto:%s\">%s</a>", tmp, tmp2, tmp2);
 			g_free(tmp);
 			g_free(tmp2);
@@ -137,7 +138,7 @@ gchar * parseAuthor(xmlNodePtr cur) {
 		}
 					
 		if (!xmlStrcmp(cur->name, BAD_CAST"url")) {
-			tmp2 = common_utf8_fix(xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
+			tmp2 = common_utf8_fix((gchar *)xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
 			tmp3 = g_strdup_printf("%s (<a href=\"%s\">Website</a>)", tmp, tmp2);
 			g_free(tmp);
 			g_free(tmp2);
@@ -192,21 +193,21 @@ static void pie_parse(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 					ctxt->title = tmp;
 				}
 			} else if(!xmlStrcmp(cur->name, BAD_CAST"link")) {
-				tmp = common_utf8_fix(xmlGetProp(cur, BAD_CAST"href"));
+				tmp = xml_get_attribute (cur, "href");
 				if(tmp) {				
 					/* 0.3 link : rel, type and href attribute */
-					tmp2 = common_utf8_fix(xmlGetProp(cur, BAD_CAST"rel"));
-					if(tmp2 && !xmlStrcmp(tmp2, BAD_CAST"alternate"))
-						feed_set_html_url(ctxt->feed, subscription_get_source(ctxt->subscription), tmp);
-					else
-						/* FIXME: Maybe do something with other links? */;
+					tmp2 = xml_get_attribute (cur, "rel");
+					if(tmp2 && g_str_equal(tmp2, "alternate"))
+						subscription_set_homepage (ctxt->subscription, tmp);
+					/* else
+						FIXME: Maybe do something with other links? */
 					g_free(tmp2);
 					g_free(tmp);
 				} else {
 					/* 0.2 link : element content is the link, or non-alternate link in 0.3 */
-					tmp = common_utf8_fix(xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1));
+					tmp = common_utf8_fix((gchar *)xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1));
 					if(tmp) {
-						feed_set_html_url(ctxt->feed, subscription_get_source(ctxt->subscription), tmp);
+						subscription_set_homepage (ctxt->subscription, tmp);
 						g_free(tmp);
 					}
 				}
@@ -226,16 +227,16 @@ static void pie_parse(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 					g_free (tmp);				
 				}
 			} else if(!xmlStrcmp(cur->name, BAD_CAST"generator")) {
-				tmp = unhtmlize(xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1));
+				tmp = unhtmlize((gchar *)xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1));
 				if(tmp && tmp[0] != '\0') {
-					tmp2 = common_utf8_fix(xmlGetProp(cur, BAD_CAST"version"));
+					tmp2 = xml_get_attribute (cur, "version");
 					if(tmp2) {
 						tmp3 = g_strdup_printf("%s %s", tmp, tmp2);
 						g_free(tmp);
 						g_free(tmp2);
 						tmp = tmp3;
 					}
-					tmp2 = common_utf8_fix(xmlGetProp(cur, BAD_CAST"url"));
+					tmp2 = xml_get_attribute (cur, "url");
 					if(tmp2) {
 						tmp3 = g_strdup_printf("<a href=\"%s\">%s</a>", tmp2, tmp);
 						g_free(tmp2);
@@ -253,7 +254,7 @@ static void pie_parse(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 				}				
 				
 			} else if(!xmlStrcmp(cur->name, BAD_CAST"modified")) { /* Modified was last used in IETF draft 02) */
-				tmp = common_utf8_fix(xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1));
+				tmp = common_utf8_fix((gchar *)xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1));
 				if(tmp) {
 					ctxt->subscription->metadata = metadata_list_append(ctxt->subscription->metadata, "pubDate", tmp);
 					ctxt->feed->time = parseISO8601Date(tmp);
@@ -261,7 +262,7 @@ static void pie_parse(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 				}
 
 			} else if(!xmlStrcmp(cur->name, BAD_CAST"updated")) { /* Updated was added in IETF draft 03 */
-				tmp = common_utf8_fix(xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1));
+				tmp = common_utf8_fix((gchar *)xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1));
 				if(tmp) {
 					ctxt->subscription->metadata = metadata_list_append(ctxt->subscription->metadata, "pubDate", tmp);
 					ctxt->feed->time = parseISO8601Date(tmp);
@@ -299,25 +300,28 @@ static gboolean pie_format_check(xmlDocPtr doc, xmlNodePtr cur) {
 		return FALSE;
 }
 
-static void pie_add_ns_handler(NsHandler *handler) {
-
-	g_assert(NULL != pie_nstable);
-	g_hash_table_insert(pie_nstable, handler->prefix, handler);
-	g_assert(handler->registerNs != NULL);
-	handler->registerNs(handler, pie_nstable, ns_pie_ns_uri_table);
+static void
+pie_add_ns_handler (NsHandler *handler)
+{
+	g_assert (NULL != pie_nstable);
+	g_hash_table_insert (pie_nstable, (gpointer)handler->prefix, handler);
+	g_assert (handler->registerNs != NULL);
+	handler->registerNs (handler, pie_nstable, ns_pie_ns_uri_table);
 }
 
-feedHandlerPtr pie_init_feed_handler(void) {
+feedHandlerPtr
+pie_init_feed_handler (void)
+{
 	feedHandlerPtr	fhp;
 	
-	fhp = g_new0(struct feedHandler, 1);
+	fhp = g_new0 (struct feedHandler, 1);
 	
-	if(NULL == pie_nstable) {
-		pie_nstable = g_hash_table_new(g_str_hash, g_str_equal);
-		ns_pie_ns_uri_table = g_hash_table_new(g_str_hash, g_str_equal);
+	if (!pie_nstable) {
+		pie_nstable = g_hash_table_new (g_str_hash, g_str_equal);
+		ns_pie_ns_uri_table = g_hash_table_new (g_str_hash, g_str_equal);
 		
 		/* register RSS name space handlers */
-		pie_add_ns_handler(ns_dc_getRSSNsHandler());
+		pie_add_ns_handler (ns_dc_get_handler ());
 	}	
 
 
