@@ -1,7 +1,7 @@
 /**
- * @file rss_channel.c some tolerant and generic RSS/RDF channel parsing
+ * @file rss_channel.c  some tolerant and generic RSS/RDF channel parsing
  *
- * Copyright (C) 2003-2007 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2009 Lars Lindner <lars.lindner@gmail.com>
  * Copyright (C) 2005-2006 Nathan Conrad <t98502@users.sourceforge.net> 
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -19,29 +19,28 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
-
-#include <sys/time.h>
 #include <string.h>
 #include <stdlib.h>
 
 #include "conf.h"
 #include "common.h"
 #include "rss_channel.h"
-#include "feed.h"
+#include "feed_parser.h"
 #include "feedlist.h"
 #include "metadata.h"
-#include "ns_dc.h"
-#include "ns_content.h"
-#include "ns_slash.h"
-#include "ns_syn.h"
 #include "ns_admin.h"
 #include "ns_ag.h"
 #include "ns_blogChannel.h"
 #include "ns_cC.h"
+#include "ns_content.h"
+#include "ns_dc.h"
+#include "ns_georss.h"
+#include "ns_itunes.h"
+#include "ns_media.h"
 #include "ns_photo.h"
+#include "ns_slash.h"
+#include "ns_syn.h"
+#include "ns_trackback.h"
 #include "ns_wfw.h"
 #include "rss_item.h"
 #include "xml.h"
@@ -89,35 +88,35 @@ static void parseChannel(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 			
 		/* Check for metadata tags */
 		if(NULL != (tmp2 = g_hash_table_lookup(RssToMetadataMapping, cur->name))) {
-			if(NULL != (tmp3 = common_utf8_fix(xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, TRUE)))) {
+			if(NULL != (tmp3 = common_utf8_fix((gchar *)xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, TRUE)))) {
 				ctxt->subscription->metadata = metadata_list_append(ctxt->subscription->metadata, tmp2, tmp3);
 				g_free(tmp3);
 			}
 		}	
 		/* check for specific tags */
 		else if(!xmlStrcmp(cur->name, BAD_CAST"pubDate")) {
- 			if(NULL != (tmp = common_utf8_fix(xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1)))) {
+ 			if(NULL != (tmp = common_utf8_fix((gchar *)xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1)))) {
 				ctxt->subscription->metadata = metadata_list_append(ctxt->subscription->metadata, "pubDate", tmp);
 				ctxt->feed->time = parseRFC822Date(tmp);
 				g_free(tmp);
 			}
 		} 
 		else if(!xmlStrcmp(cur->name, BAD_CAST"ttl")) {
- 			if(NULL != (tmp = common_utf8_fix(xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, TRUE)))) {
+ 			if(NULL != (tmp = common_utf8_fix((gchar *)xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, TRUE)))) {
 				subscription_set_default_update_interval(ctxt->subscription, atoi(tmp));
 				g_free(tmp);
 			}
 		}
 		else if(!xmlStrcmp(cur->name, BAD_CAST"title")) {
- 			if(NULL != (tmp = unhtmlize(xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, TRUE)))) {
+ 			if(NULL != (tmp = unhtmlize((gchar *)xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, TRUE)))) {
 				if(ctxt->title)
 					g_free(ctxt->title);
 				ctxt->title = tmp;
 			}
 		}
 		else if(!xmlStrcmp(cur->name, BAD_CAST"link")) {
- 			if(NULL != (tmp = unhtmlize(xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, TRUE)))) {
-				feed_set_html_url(ctxt->feed, subscription_get_source(ctxt->subscription), tmp);
+ 			if(NULL != (tmp = unhtmlize((gchar *)xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, TRUE)))) {
+				subscription_set_homepage (ctxt->subscription, tmp);
 				g_free(tmp);
 			}
 		}
@@ -149,10 +148,10 @@ static gchar* parseTextInput(xmlNodePtr cur) {
 				tiDescription = xhtml_extract (cur, 0, NULL);
 			} else if(!xmlStrcmp(cur->name, BAD_CAST"name")) {
 				g_free(tiName);
-				tiName = common_utf8_fix(xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
+				tiName = common_utf8_fix((gchar *)xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
 			} else if(!xmlStrcmp(cur->name, BAD_CAST"link")) {
 				g_free(tiLink);
-				tiLink = common_utf8_fix(xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
+				tiLink = common_utf8_fix((gchar *)xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
 			}
 		}
 		cur = cur->next;
@@ -189,7 +188,7 @@ static gchar* parseImage(xmlNodePtr cur) {
 	while (cur != NULL) {
 		if (cur->type == XML_ELEMENT_NODE) {
 			if (!xmlStrcmp(cur->name, BAD_CAST"url")) {
-				tmp = common_utf8_fix(xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
+				tmp = common_utf8_fix((gchar *)xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
 				if(NULL != tmp) {
 					return tmp;
 				}
@@ -264,7 +263,7 @@ static void rss_parse(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 			/* save link to channel image */
 			if((!xmlStrcmp(cur->name, BAD_CAST"image"))) {
 				if(NULL != (tmp = parseImage(cur))) {
-					feed_set_image_url(ctxt->feed, tmp);
+					metadata_list_set (&ctxt->subscription->metadata, "imageUrl", tmp);
 					g_free(tmp);
 				}
 				
@@ -320,52 +319,59 @@ static gboolean rss_format_check(xmlDocPtr doc, xmlNodePtr cur) {
 	return FALSE;
 }
 
-static void rss_add_ns_handler(NsHandler *handler) {
-
-	g_assert(NULL != rss_nstable);
-	g_hash_table_insert(rss_nstable, handler->prefix, handler);
-	g_assert(handler->registerNs != NULL);
-	handler->registerNs(handler, rss_nstable, ns_rss_ns_uri_table);
+static void
+rss_add_ns_handler (NsHandler *handler)
+{
+	g_assert (NULL != rss_nstable);
+	g_hash_table_insert (rss_nstable, (gpointer)handler->prefix, handler);
+	g_assert (handler->registerNs != NULL);
+	handler->registerNs (handler, rss_nstable, ns_rss_ns_uri_table);
 }
 
-feedHandlerPtr rss_init_feed_handler(void) {
+feedHandlerPtr
+rss_init_feed_handler (void)
+{
 	feedHandlerPtr	fhp;
 	
-	fhp = g_new0(struct feedHandler, 1);
+	fhp = g_new0 (struct feedHandler, 1);
 
 	/* Note: the tag mapping definitions and namespace registration
 	   infos are shared with rss_item.c */
 	
-	if(RssToMetadataMapping == NULL) {
-		RssToMetadataMapping = g_hash_table_new(g_str_hash, g_str_equal);
-		g_hash_table_insert(RssToMetadataMapping, "copyright", "copyright");
-		g_hash_table_insert(RssToMetadataMapping, "category", "category");
-		g_hash_table_insert(RssToMetadataMapping, "webMaster", "webmaster");
-		g_hash_table_insert(RssToMetadataMapping, "language", "language");
-		g_hash_table_insert(RssToMetadataMapping, "managingEditor", "managingEditor");
-		g_hash_table_insert(RssToMetadataMapping, "lastBuildDate", "contentUpdateDate");
-		g_hash_table_insert(RssToMetadataMapping, "generator", "feedgenerator");
-		g_hash_table_insert(RssToMetadataMapping, "publisher", "webmaster");
-		g_hash_table_insert(RssToMetadataMapping, "author", "author");
-		g_hash_table_insert(RssToMetadataMapping, "comments", "commentsUri");
+	if (!RssToMetadataMapping) {
+		RssToMetadataMapping = g_hash_table_new (g_str_hash, g_str_equal);
+		g_hash_table_insert (RssToMetadataMapping, "copyright", "copyright");
+		g_hash_table_insert (RssToMetadataMapping, "category", "category");
+		g_hash_table_insert (RssToMetadataMapping, "webMaster", "webmaster");
+		g_hash_table_insert (RssToMetadataMapping, "language", "language");
+		g_hash_table_insert (RssToMetadataMapping, "managingEditor", "managingEditor");
+		g_hash_table_insert (RssToMetadataMapping, "lastBuildDate", "contentUpdateDate");
+		g_hash_table_insert (RssToMetadataMapping, "generator", "feedgenerator");
+		g_hash_table_insert (RssToMetadataMapping, "publisher", "webmaster");
+		g_hash_table_insert (RssToMetadataMapping, "author", "author");
+		g_hash_table_insert (RssToMetadataMapping, "comments", "commentsUri");
 	}
 	
-	if(NULL == rss_nstable) {
-		rss_nstable = g_hash_table_new(g_str_hash, g_str_equal);
-		ns_rss_ns_uri_table = g_hash_table_new(g_str_hash, g_str_equal);
-		
-		/* register RSS name space handlers */
-		rss_add_ns_handler(ns_bC_getRSSNsHandler());
-		rss_add_ns_handler(ns_dc_getRSSNsHandler());
-  		rss_add_ns_handler(ns_slash_getRSSNsHandler());
-		rss_add_ns_handler(ns_content_getRSSNsHandler());
-		rss_add_ns_handler(ns_syn_getRSSNsHandler());
-		rss_add_ns_handler(ns_admin_getRSSNsHandler());
-		rss_add_ns_handler(ns_ag_getRSSNsHandler());
-		rss_add_ns_handler(ns_cC_getRSSNsHandler());
-		rss_add_ns_handler(ns_photo_getRSSNsHandler());
-		rss_add_ns_handler(ns_pb_getRSSNsHandler());
-		rss_add_ns_handler(ns_wfw_getRSSNsHandler());
+	if (!rss_nstable) {
+		rss_nstable = g_hash_table_new (g_str_hash, g_str_equal);
+		ns_rss_ns_uri_table = g_hash_table_new (g_str_hash, g_str_equal);
+
+		/* register name space handlers */		
+		rss_add_ns_handler (ns_bC_get_handler ());
+		rss_add_ns_handler (ns_dc_get_handler ());
+  		rss_add_ns_handler (ns_slash_get_handler ());
+		rss_add_ns_handler (ns_content_get_handler ());
+		rss_add_ns_handler (ns_syn_get_handler ());
+		rss_add_ns_handler (ns_admin_get_handler ());
+		rss_add_ns_handler (ns_ag_get_handler ());
+		rss_add_ns_handler (ns_cC_get_handler ());
+		rss_add_ns_handler (ns_photo_get_handler ());
+		rss_add_ns_handler (ns_pb_get_handler ());
+		rss_add_ns_handler (ns_wfw_get_handler ());
+		rss_add_ns_handler (ns_media_get_handler ());
+		rss_add_ns_handler (ns_itunes_get_handler ());
+		rss_add_ns_handler (ns_trackback_get_handler ());
+		rss_add_ns_handler (ns_georss_get_handler ());
 	}
 							
 	/* prepare feed handler structure */
