@@ -1,7 +1,7 @@
 /**
- * @file rss_item.c RSS/RDF item parsing 
+ * @file rss_item.c  RSS/RDF item parsing 
  *
- * Copyright (C) 2003-2007 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2008 Lars Lindner <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,13 +18,10 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
-
 #include <string.h>
 
 #include "common.h"
+#include "enclosure.h"
 #include "rss_item.h"
 #include "metadata.h"
 #include "xml.h"
@@ -38,35 +35,39 @@ extern GHashTable	*rss_nstable;
 extern GHashTable	*ns_rss_ns_uri_table;
 
 /* method to parse standard tags for each item element */
-itemPtr parseRSSItem(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
+itemPtr
+parseRSSItem (feedParserCtxtPtr ctxt, xmlNodePtr cur)
+{
 	gchar			*tmp, *tmp2, *tmp3;
 	NsHandler		*nsh;
 	parseItemTagFunc	pf;
 	
 	g_assert(NULL != cur);
 
-	ctxt->item = item_new();
-	ctxt->item->tmpdata = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, g_free);
+	ctxt->item = item_new ();
+	ctxt->item->tmpdata = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, g_free);
 	
 	/* try to get an item about id */
-	if(tmp = xmlGetProp(cur, BAD_CAST"about")) {
-		item_set_id(ctxt->item, tmp);
-		item_set_source(ctxt->item, tmp);
-		g_free(tmp);
+	tmp = xml_get_attribute (cur, "about");
+	if (tmp) {
+		item_set_id (ctxt->item, tmp);
+		item_set_source (ctxt->item, tmp);
+		g_free (tmp);
 	}
 	
 	cur = cur->xmlChildrenNode;
-	while(cur) {
-		if(cur->type != XML_ELEMENT_NODE || !cur->name) {
+	while (cur) {
+		if (cur->type != XML_ELEMENT_NODE || !cur->name) {
 			cur = cur->next;
 			continue;
 		}
 		
 		/* check namespace of this tag */
-		if(cur->ns) {
+		if (cur->ns) {
 			if((cur->ns->href && (nsh = (NsHandler *)g_hash_table_lookup(ns_rss_ns_uri_table, (gpointer)cur->ns->href))) ||
 			   (cur->ns->prefix && (nsh = (NsHandler *)g_hash_table_lookup(rss_nstable, (gpointer)cur->ns->prefix)))) {
-				if(pf = nsh->parseItemTag)
+				pf = nsh->parseItemTag;
+				if (pf)
 					(*pf)(ctxt, cur);
 				cur = cur->next;
 				continue;
@@ -76,22 +77,33 @@ itemPtr parseRSSItem(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 		} /* explicitly no following else!!! */
 		
 		/* check for metadata tags */
-		if(tmp2 = g_hash_table_lookup(RssToMetadataMapping, cur->name)) {
-			if(tmp3 = common_utf8_fix(xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, TRUE))) {
+		tmp2 = g_hash_table_lookup(RssToMetadataMapping, cur->name);
+		if (tmp2) {
+			tmp3 = common_utf8_fix((gchar *)xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, TRUE));
+			if (tmp3) {
 				ctxt->item->metadata = metadata_list_append(ctxt->item->metadata, tmp2, tmp3);
 				g_free(tmp3);
 			}
 		}
 		/* check for specific tags */
 		else if(!xmlStrcmp(cur->name, BAD_CAST"pubDate")) {
- 			if(tmp = common_utf8_fix(xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1))) {
+ 			tmp = common_utf8_fix((gchar *)xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1));
+			if (tmp) {
 				ctxt->item->time = parseRFC822Date(tmp);
 				g_free(tmp);
 			}
 		} 
 		else if(!xmlStrcmp(cur->name, BAD_CAST"enclosure")) {
 			/* RSS 0.93 allows multiple enclosures */
-			if(tmp = common_utf8_fix(xmlGetProp(cur, BAD_CAST"url"))) {
+			tmp = xml_get_attribute (cur, "url");
+			if (tmp) {
+				gchar *type = xml_get_attribute (cur, "type");
+				gchar *lengthStr = xml_get_attribute (cur, "length");
+				gchar *enclStr = NULL;
+				gssize length = 0;
+				if (lengthStr)
+					length = atol (lengthStr);
+				
 				if((strstr(tmp, "://") == NULL) &&
 				   (ctxt->feed->htmlUrl != NULL) &&
 				   (ctxt->feed->htmlUrl[0] != '|') &&
@@ -102,39 +114,50 @@ itemPtr parseRSSItem(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 					 tmp = tmp2;
 				}
 		
-				ctxt->item->metadata = metadata_list_append(ctxt->item->metadata, "enclosure", tmp);
+				enclStr = enclosure_values_to_string (tmp, type, length, FALSE);
+				ctxt->item->metadata = metadata_list_append(ctxt->item->metadata, "enclosure", enclStr);
 				ctxt->item->hasEnclosure = TRUE;
-				g_free(tmp);
+
+				g_free (enclStr);
+				g_free (tmp);
+				g_free (type);
+				g_free (lengthStr);
 			}
 		} 
 		else if(!xmlStrcmp(cur->name, BAD_CAST"guid")) {
 			if(!item_get_id(ctxt->item)) {
-				if(tmp = xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1)) {
-					item_set_id(ctxt->item, tmp);
-					ctxt->item->validGuid = TRUE;
-					tmp2 = xmlGetProp(cur, "isPermaLink");
-					if(!item_get_source(ctxt->item) && (tmp2 == NULL || !xmlStrcmp(tmp2, BAD_CAST"true")))
-						item_set_source(ctxt->item, tmp); /* Per the RSS 2.0 spec. */
-					if(tmp2)
-						xmlFree(tmp2);
+				tmp = (gchar *)xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1);
+				if (tmp) {
+					if (strlen (tmp) > 0) {
+						item_set_id(ctxt->item, tmp);
+						ctxt->item->validGuid = TRUE;
+						tmp2 = xml_get_attribute (cur, "isPermaLink");
+						if(!item_get_source(ctxt->item) && (tmp2 == NULL || g_str_equal (tmp2, "true")))
+							item_set_source(ctxt->item, tmp); /* Per the RSS 2.0 spec. */
+						if(tmp2)
+							xmlFree(tmp2);
+					}
 					xmlFree(tmp);
 				}
 			}
 		}
 		else if(!xmlStrcmp(cur->name, BAD_CAST"title")) {
- 			if(tmp = unhtmlize(xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, TRUE))) {
+ 			tmp = unhtmlize((gchar *)xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, TRUE));
+			if (tmp) {
 				item_set_title(ctxt->item, tmp);
 				g_free(tmp);
 			}
 		}
 		else if(!xmlStrcmp(cur->name, BAD_CAST"link")) {
- 			if(tmp = unhtmlize(xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, TRUE))) {
+ 			tmp = unhtmlize((gchar *)xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, TRUE));
+			if (tmp) {
 				item_set_source(ctxt->item, tmp);
 				g_free(tmp);
 			}
 		}
 		else if(!xmlStrcmp(cur->name, BAD_CAST"description")) {
- 			if(tmp = common_utf8_fix(xhtml_extract (cur, 0, NULL))) {
+ 			tmp = common_utf8_fix(xhtml_extract (cur, 0, NULL));
+			if (tmp) {
 				/* don't overwrite content:encoded descriptions... */
 				if(!item_get_description(ctxt->item))
 					item_set_description(ctxt->item, tmp);
@@ -142,12 +165,14 @@ itemPtr parseRSSItem(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 			}
 		}
 		else if(!xmlStrcmp(cur->name, BAD_CAST"source")) {
-			if(tmp = common_utf8_fix(xmlGetProp(cur, BAD_CAST"url"))) {
-				item_set_real_source_url(ctxt->item, tmp);
-				g_free(tmp);
+			tmp = xml_get_attribute (cur, "url");
+			if (tmp) {
+				metadata_list_set (&(ctxt->item->metadata), "realSourceUrl", g_strchomp (tmp));
+				g_free (tmp);
 			}
-			if(tmp = unhtmlize(xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1))) {
-				item_set_real_source_title(ctxt->item, tmp);
+			tmp = unhtmlize ((gchar *)xmlNodeListGetString (ctxt->doc, cur->xmlChildrenNode, 1));
+			if (tmp) {
+				metadata_list_set (&(ctxt->item->metadata), "realSourceTitle", g_strchomp (tmp));
 				g_free(tmp);
 			}
 		}
