@@ -1,7 +1,7 @@
 /**
- * @file rule.c DB based item matching rule handling
+ * @file rule.c  DB based item matching rule handling
  *
- * Copyright (C) 2003-2007 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2008 Lars Lindner <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,6 +48,10 @@ typedef gboolean (*ruleCheckFunc)	(rulePtr rule, itemPtr item);
 struct ruleInfo *ruleFunctions = NULL;
 gint nrOfRuleFunctions = 0;
 
+static void rule_init (void);
+
+/* rule creation */
+
 rulePtr
 rule_new (struct vfolder *vfolder,
           const gchar *ruleId,
@@ -58,6 +62,9 @@ rule_new (struct vfolder *vfolder,
 	rulePtr		rule;
 	int		i;
 	
+	if (!ruleFunctions)
+		rule_init ();
+	
 	for (i = 0, ruleInfo = ruleFunctions; i < nrOfRuleFunctions; i++, ruleInfo++) 
 	{
 		if (0 == strcmp (ruleInfo->ruleId, ruleId)) 
@@ -80,16 +87,14 @@ rule_free (rulePtr rule)
 	g_free (rule);
 }
 
-/* -------------------------------------------------------------------- */
-/* rule conditions							*/
-/* -------------------------------------------------------------------- */
+/* rule conditions */
 
 static conditionPtr
 rule_condition_feed_title_match (rulePtr rule) 
 {
 	conditionPtr	condition;
 
-	condition = g_new0(struct condition, 1);
+	condition = g_new0 (struct condition, 1);
 	condition->tables = QUERY_TABLE_NODE;
 	condition->sql = g_strdup_printf ("node.title LIKE '%%%s%%'", rule->value);
 	
@@ -101,7 +106,7 @@ rule_condition_item_title_match (rulePtr rule)
 {
 	conditionPtr	condition;
 
-	condition = g_new0(struct condition, 1);
+	condition = g_new0 (struct condition, 1);
 	condition->tables = QUERY_TABLE_ITEMS;
 	condition->sql = g_strdup_printf ("items.title LIKE '%%%s%%'", rule->value);
 	
@@ -113,7 +118,7 @@ rule_condition_item_description_match (rulePtr rule)
 {
 	conditionPtr	condition;
 	
-	condition = g_new0(struct condition, 1);
+	condition = g_new0 (struct condition, 1);
 	condition->tables = QUERY_TABLE_ITEMS;
 	condition->sql = g_strdup_printf ("items.description LIKE '%%%s%%'", rule->value);
 	
@@ -125,7 +130,7 @@ rule_condition_item_match (rulePtr rule)
 {
 	conditionPtr	condition;
 	
-	condition = g_new0(struct condition, 1);
+	condition = g_new0 (struct condition, 1);
 	condition->tables = QUERY_TABLE_ITEMS;
 	condition->sql = g_strdup_printf ("(items.title LIKE '%%%s%%' OR items.description LIKE '%%%s%%')", rule->value, rule->value);
 	
@@ -137,7 +142,7 @@ rule_condition_item_is_unread (rulePtr rule)
 {
 	conditionPtr	condition;
 	
-	condition = g_new0(struct condition, 1);
+	condition = g_new0 (struct condition, 1);
 	condition->tables = QUERY_TABLE_ITEMS;
 	condition->sql = g_strdup ("items.read = 0");
 	
@@ -155,7 +160,7 @@ rule_condition_item_is_flagged (rulePtr rule)
 {
 	conditionPtr	condition;
 	
-	condition = g_new0(struct condition, 1);
+	condition = g_new0 (struct condition, 1);
 	condition->tables = QUERY_TABLE_ITEMS;
 	condition->sql = g_strdup ("items.marked = 1");
 	
@@ -173,7 +178,7 @@ rule_condition_item_was_updated (rulePtr rule)
 {
 	conditionPtr	condition;
 	
-	condition = g_new0(struct condition, 1);
+	condition = g_new0 (struct condition, 1);
 	condition->tables = QUERY_TABLE_ITEMS;
 	condition->sql = g_strdup ("items.updated = 1");
 	
@@ -192,6 +197,18 @@ rule_condition_item_has_enclosure (rulePtr rule)
 	return condition;
 }
 
+static conditionPtr
+rule_condition_item_has_category (rulePtr rule)
+{
+	conditionPtr	condition;
+	
+	condition = g_new0 (struct condition, 1);
+	condition->tables = QUERY_TABLE_METADATA | QUERY_TABLE_ITEMS;
+	condition->sql = g_strdup_printf ("(metadata.key = 'category' AND metadata.value = '%s')", rule->value);
+	
+	return condition;
+}
+
 static void
 rule_merge_sql (gchar **sql, const gchar *operator, const gchar *condition)
 {
@@ -205,10 +222,9 @@ rule_merge_sql (gchar **sql, const gchar *operator, const gchar *condition)
 }
 
 static queryPtr
-query_create (GSList *rules)
+query_create (GSList *rules, gboolean anyMatch)
 {
 	queryPtr	query;
-	conditionPtr	condition;
 	GSList		*iter;
 	GSList		*additive = NULL;
 	GSList		*negative = NULL;
@@ -242,11 +258,11 @@ query_create (GSList *rules)
 	
 	/* Constructing query by ... */
 	  
-	/* a) OR'ing all additive matches */
+	/* a) joining all additive matches (OR'ing and AND'ing according to "any or all rules matching" parameter) */
 	iter = additive;
 	while (iter) {
 		gchar *sql = (gchar *)iter->data;
-		rule_merge_sql (&query->conditions, "OR", sql);
+		rule_merge_sql (&query->conditions, anyMatch?"OR":"AND", sql);
 		g_free (sql);
 		iter = g_slist_next (iter);
 	}
@@ -279,27 +295,27 @@ query_free (queryPtr query)
 }
 
 void
-rules_to_view (GSList *rules, const gchar *id)
+rules_to_view (GSList *rules, gboolean anyMatch, const gchar *id)
 {
 	queryPtr	query;
 
 	if (0 == g_slist_length (rules))
 		return;	
 		
-	query = query_create (rules);
+	query = query_create (rules, anyMatch);
 	query->columns = QUERY_COLUMN_ITEM_ID | QUERY_COLUMN_ITEM_READ_STATUS;
 	db_view_create (id, query);
 	query_free (query);
 }
 
 gboolean
-rules_check_item (GSList *rules, itemPtr item)
+rules_check_item (GSList *rules, gboolean anyMatch, itemPtr item)
 {
 	gboolean	result;
 	queryPtr	query;
 	
 	if (0 == g_slist_length (rules))
-		return;
+		return FALSE;
 	
 	/* first try in memory checks (for "unread" and "important" search folder)... */
 	if (1 == g_slist_length (rules)) {
@@ -312,7 +328,7 @@ rules_check_item (GSList *rules, itemPtr item)
 	}
 
 	/* if not possible query DB */
-	query = query_create (rules);	
+	query = query_create (rules, anyMatch);	
 	query->columns = QUERY_COLUMN_ITEM_ID;
 	result = db_item_check (item->id, query);
 	query_free (query);
@@ -325,7 +341,7 @@ rules_check_item (GSList *rules, itemPtr item)
 static void
 rule_add (ruleConditionFunc queryFunc,
           ruleCheckFunc checkFunc,
-          gchar *ruleId, 
+          const gchar *ruleId, 
           gchar *title,
           gchar *positive,
           gchar *negative,
@@ -346,19 +362,23 @@ rule_add (ruleConditionFunc queryFunc,
 	nrOfRuleFunctions++;
 }
 
-void
+static void
 rule_init (void) 
 {
 	debug_enter ("rule_init");
 
+	/*        SQL condition builder function	in-memory check function	feedlist.opml rule id           rule menu label         positive menu option    negative menu option    has param */ 
+	/*        ========================================================================================================================================================================================*/
+	
 	rule_add (rule_condition_item_match,		NULL,				ITEM_MATCH_RULE_ID,		_("Item"),		_("does contain"),	_("does not contain"),	TRUE);
-	rule_add (rule_condition_item_title_match,	NULL,				ITEM_TITLE_MATCH_RULE_ID,	_("Item title"),	_("does match"),	_("does not match"),	TRUE);
-	rule_add (rule_condition_item_description_match, NULL,				ITEM_DESC_MATCH_RULE_ID,	_("Item body"),		_("does match"),	_("does not match"),	TRUE);
-	rule_add (rule_condition_feed_title_match,	NULL,				"feed_title",			_("Feed title"),	_("does match"),	_("does not match"),	TRUE);
+	rule_add (rule_condition_item_title_match,	NULL,				ITEM_TITLE_MATCH_RULE_ID,	_("Item title"),	_("does contain"),	_("does not contain"),	TRUE);
+	rule_add (rule_condition_item_description_match, NULL,				ITEM_DESC_MATCH_RULE_ID,	_("Item body"),		_("does contain"),	_("does not contain"),	TRUE);
+	rule_add (rule_condition_feed_title_match,	NULL,				"feed_title",			_("Feed title"),	_("does contain"),	_("does not contain"),	TRUE);
 	rule_add (rule_condition_item_is_unread,	rule_check_item_is_unread,	"unread",			_("Read status"),	_("is unread"),		_("is read"),		FALSE);
 	rule_add (rule_condition_item_is_flagged,	rule_check_item_is_flagged,	"flagged",			_("Flag status"),	_("is flagged"),	_("is unflagged"),	FALSE);
 	rule_add (rule_condition_item_was_updated,	NULL, 				"updated",			_("Update status"),	_("was updated"),	_("was not updated"),	FALSE);
 	rule_add (rule_condition_item_has_enclosure,	NULL,				"enclosure",			_("Podcast"),		_("included"),		_("not included"),	FALSE);
+	rule_add (rule_condition_item_has_category,	NULL,				"category",			_("Category"),		_("is set"),		_("is not set"),	TRUE);
 
 	debug_exit ("rule_init");
 }
