@@ -1,7 +1,7 @@
 /**
  * @file subscription.c common subscription handling
  * 
- * Copyright (C) 2003-2007 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2008 Lars Lindner <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,8 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include "subscription.h"
+
 #include <string.h>
 
 #include "common.h"
@@ -27,9 +29,10 @@
 #include "favicon.h"
 #include "feedlist.h"
 #include "metadata.h"
-#include "subscription.h"
-#include "net/cookies.h"
-#include "ui/ui_mainwindow.h"
+#include "net.h"
+#include "ui/auth_dialog.h"
+#include "ui/itemview.h"
+#include "ui/liferea_shell.h"
 #include "ui/ui_node.h"
 
 /* The allowed feed protocol prefixes (see http://25hoursaday.com/draft-obasanjo-feed-URI-scheme-02.html) */
@@ -44,6 +47,7 @@ subscription_new (const gchar *source,
 	subscriptionPtr	subscription;
 	
 	subscription = g_new0 (struct subscription, 1);
+	subscription->type = feed_get_subscription_type ();
 	subscription->updateOptions = options;
 	
 	if (!subscription->updateOptions)
@@ -97,12 +101,12 @@ gboolean
 subscription_can_be_updated (subscriptionPtr subscription)
 {
 	if (subscription->updateJob) {
-		ui_mainwindow_set_status_bar (_("Subscription \"%s\" is already being updated!"), node_get_title (subscription->node));
+		liferea_shell_set_status_bar (_("Subscription \"%s\" is already being updated!"), node_get_title (subscription->node));
 		return FALSE;
 	}
 	
 	if (subscription->discontinued) {
-		ui_mainwindow_set_status_bar (_("The subscription \"%s\" was discontinued. Liferea won't update it anymore!"), node_get_title (subscription->node));
+		liferea_shell_set_status_bar (_("The subscription \"%s\" was discontinued. Liferea won't update it anymore!"), node_get_title (subscription->node));
 		return FALSE;
 	}
 
@@ -137,7 +141,7 @@ void
 subscription_update_favicon (subscriptionPtr subscription)
 {
 	debug1 (DEBUG_UPDATE, "trying to download favicon.ico for \"%s\"", node_get_title (subscription->node));
-	ui_mainwindow_set_status_bar (_("Updating favicon for \"%s\""), node_get_title (subscription->node));
+	liferea_shell_set_status_bar (_("Updating favicon for \"%s\""), node_get_title (subscription->node));
 	g_get_current_time (&subscription->updateState->lastFaviconPoll);
 	db_update_state_save (subscription->node->id, subscription->updateState);
 	favicon_download (subscription->node->id,
@@ -148,62 +152,93 @@ subscription_update_favicon (subscriptionPtr subscription)
 			  (gpointer)subscription->node);
 }
 
-typedef struct subscriptionUpdateCtxt {
-	subscriptionPtr		subscription;
-	subscription_update_cb	callback;
-} *subscriptionUpdateCtxtPtr;
+/**
+ * Updates the error status of the given subscription
+ *
+ * @param subscription	the subscription
+ * @param httpstatus	the new HTTP status code
+ * @param resultcode	the update result code
+ * @param filterError	filter error string (or NULL)
+ */
+static void
+subscription_update_error_status (subscriptionPtr subscription,
+                                  gint httpstatus,
+                                  gint resultcode,
+                                  gchar *filterError)
+{
+	gboolean	errorFound = FALSE;
+
+	if (subscription->filterError)
+		g_free (subscription->filterError);
+	if (subscription->httpError)
+		g_free (subscription->httpError);
+	if (subscription->updateError)
+		g_free (subscription->updateError);
+
+	subscription->filterError = g_strdup (filterError);
+	subscription->updateError = NULL;
+	subscription->httpError = NULL;
+	subscription->httpErrorCode = httpstatus;
+
+	if (((httpstatus >= 200) && (httpstatus < 400)) && /* HTTP codes starting with 2 and 3 mean no error */
+	    (NULL == subscription->filterError))
+		return;
+
+	if ((200 != httpstatus) || (resultcode != 0)) {
+		subscription->httpError = g_strdup (network_strerror (resultcode, httpstatus));
+		errorFound = TRUE;
+	}
+
+	/* if none of the above error descriptions matched... */
+	if (!errorFound)
+		subscription->updateError = g_strdup (_("There was a problem while reading this subscription. Please check the URL and console output."));
+}
 
 static void
 subscription_process_update_result (const struct updateResult * const result, gpointer user_data, guint32 flags)
 {
-	subscriptionUpdateCtxtPtr ctxt = (subscriptionUpdateCtxtPtr)user_data;
-	subscriptionPtr	subscription = ctxt->subscription;
+	subscriptionPtr subscription = (subscriptionPtr)user_data;
 	nodePtr		node = subscription->node;
 	gboolean	processing = FALSE;
-	
+	GTimeVal	now;
+
 	/* 1. preprocessing */
 
 	/* update the subscription URL on permanent redirects */
 	if (result->source && g_str_equal (result->source, subscription_get_source (subscription))) {
 		subscription_set_source (subscription, result->source);
-		ui_mainwindow_set_status_bar (_("The URL of \"%s\" has changed permanently and was updated"), node_get_title(node));
+		liferea_shell_set_status_bar (_("The URL of \"%s\" has changed permanently and was updated"), node_get_title(node));
 	}
 
 	if (401 == result->httpstatus) { /* unauthorized */
-		if (flags & FEED_REQ_AUTH_DIALOG)
-			ui_auth_dialog_new (subscription, flags);
+		ui_auth_dialog_new (subscription, flags);
 	} else if (410 == result->httpstatus) { /* gone */
 		subscription->discontinued = TRUE;
 		node->available = TRUE;
-		ui_mainwindow_set_status_bar (_("\"%s\" is discontinued. Liferea won't updated it anymore!"), node_get_title (node));
+		liferea_shell_set_status_bar (_("\"%s\" is discontinued. Liferea won't updated it anymore!"), node_get_title (node));
 	} else if (304 == result->httpstatus) {
 		node->available = TRUE;
-		ui_mainwindow_set_status_bar (_("\"%s\" has not changed since last update"), node_get_title(node));
+		liferea_shell_set_status_bar (_("\"%s\" has not changed since last update"), node_get_title(node));
 	} else {
 		processing = TRUE;
 	}
 
 	subscription_update_error_status (subscription, result->httpstatus, result->returncode, result->filterErrors);
 
-	/* 2. call subscription/node type specific processing */
-	if (processing) {
-		if (ctxt->callback) {
-			(*ctxt->callback) (node, result, flags);
-		} else {
-			node_process_update_result (node, result, flags);
-		}
-	}
+	/* 2. call subscription type specific processing */
+	if (processing)
+		SUBSCRIPTION_TYPE (subscription)->process_update_result (subscription, result, flags);
 
 	/* 3. call favicon updating after subscription processing
 	      to ensure we have valid baseUrl for feed nodes... */
-	if (flags & FEED_REQ_DOWNLOAD_FAVICON)
+	g_get_current_time (&now);
+	if (favicon_update_needed (subscription->node->id, subscription->updateState, &now))
 		subscription_update_favicon (subscription);
 	
 	/* 4. generic postprocessing */
 	subscription->updateJob = NULL;
 
 	update_state_set_lastmodified (subscription->updateState, update_state_get_lastmodified (result->updateState));
-	update_state_set_etag (subscription->updateState, update_state_get_etag (result->updateState));
 	update_state_set_cookies (subscription->updateState, update_state_get_cookies (result->updateState));
 	g_get_current_time (&subscription->updateState->lastPoll);
 	db_update_state_save (subscription->node->id, subscription->updateState);
@@ -212,35 +247,24 @@ subscription_process_update_result (const struct updateResult * const result, gp
 	itemview_update ();
 	ui_node_update (subscription->node->id);
 	feedlist_schedule_save ();
-	
-	g_free (ctxt);
 }
 
 void
-subscription_update_with_callback (subscriptionPtr subscription, subscription_update_cb callback, guint flags)
+subscription_update (subscriptionPtr subscription, guint flags)
 {
-	subscriptionUpdateCtxtPtr	ctxt;
 	updateRequestPtr		request;
 	GTimeVal			now;
 	
 	if (!subscription)
 		return;
+		
+	if (subscription->updateJob)
+		return;
 	
 	debug1 (DEBUG_UPDATE, "Scheduling %s to be updated", node_get_title (subscription->node));
-	
-	ctxt = g_new0 (struct subscriptionUpdateCtxt, 1);
-	ctxt->subscription = subscription;
-	ctxt->callback = callback;
-	
-	/* Retries that might have long timeouts must be 
-	   cancelled to immediately execute the user request. */
-	if (subscription->updateJob) {
-		update_job_cancel_retry (subscription->updateJob);
-		subscription->updateJob = NULL;
-	}
 	 
 	if (subscription_can_be_updated (subscription)) {
-		ui_mainwindow_set_status_bar (_("Updating \"%s\""), node_get_title (subscription->node));
+		liferea_shell_set_status_bar (_("Updating \"%s\""), node_get_title (subscription->node));
 
 		g_get_current_time (&now);
 		subscription_reset_update_counter (subscription, &now);
@@ -253,15 +277,12 @@ subscription_update_with_callback (subscriptionPtr subscription, subscription_up
 
 		if (subscription_get_filter (subscription))
 			request->filtercmd = g_strdup (subscription_get_filter (subscription));
-		
-		subscription->updateJob = update_execute_request (subscription, request, subscription_process_update_result, ctxt, flags);
-	}
-}
 
-void
-subscription_update (subscriptionPtr subscription, guint flags)
-{
-	subscription_update_with_callback (subscription, NULL, flags);
+		if (SUBSCRIPTION_TYPE (subscription)->prepare_update_request (subscription, request))
+			subscription->updateJob = update_execute_request (subscription, request, subscription_process_update_result, subscription, flags);
+		else
+			update_request_free (request);
+	}
 }
 
 void
@@ -286,9 +307,6 @@ subscription_auto_update (subscriptionPtr subscription)
 
 	g_get_current_time (&now);
 	
-	if (favicon_update_needed (subscription->node->id, subscription->updateState, &now))
-		flags |= FEED_REQ_DOWNLOAD_FAVICON;
-			
 	if (subscription->updateState->lastPoll.tv_sec + interval*60 <= now.tv_sec)
 		subscription_update (subscription, flags);
 }
@@ -296,7 +314,9 @@ subscription_auto_update (subscriptionPtr subscription)
 void
 subscription_cancel_update (subscriptionPtr subscription)
 {
-	g_assert (subscription->updateJob);
+	if (!subscription->updateJob)
+		return;
+
 	update_job_cancel_by_owner (subscription);
 	subscription->updateJob = NULL;
 }
@@ -346,23 +366,18 @@ subscription_get_source (subscriptionPtr subscription)
 	return subscription->source;
 }
 
+const gchar *
+subscription_get_homepage (subscriptionPtr subscription)
+{
+	return metadata_list_get (subscription->metadata, "homepage");
+}
+
 const gchar *
 subscription_get_filter (subscriptionPtr subscription)
 {
 	return subscription->filtercmd;
 }
 
-void
-subscription_set_cookies (subscriptionPtr subscription, const gchar *cookies)
-{
-	g_free (subscription->updateState->cookies);
-
-	if (cookies)
-		subscription->updateState->cookies = g_strdup(cookies);
-	else
-		subscription->updateState->cookies = NULL;
-}
-
 void
 subscription_set_orig_source (subscriptionPtr subscription, const gchar *source)
 {
@@ -377,17 +392,40 @@ subscription_set_source (subscriptionPtr subscription, const gchar *source)
 	g_free (subscription->source);
 	subscription->source = g_strchomp (g_strdup (source));
 	feedlist_schedule_save ();
-	
-	if ('|' != source[0])
-		/* check if we've got matching cookies ... */
-		subscription_set_cookies (subscription, cookies_find_matching (source));
-	else 
-		subscription_set_cookies (subscription, NULL);
-	
+
+	update_state_set_cookies (subscription->updateState, NULL);
+
 	if (NULL == subscription_get_orig_source (subscription))
 		subscription_set_orig_source (subscription, source);
 }
 
+void
+subscription_set_homepage (subscriptionPtr subscription, const gchar *newHtmlUrl)
+{
+	gchar 	*htmlUrl = NULL;
+	
+	if (newHtmlUrl) {
+		if (strstr (newHtmlUrl, "://")) {
+			/* absolute URI can be used directly */
+			htmlUrl = g_strchomp (g_strdup (newHtmlUrl));
+		} else {
+			/* relative URI part needs to be expanded */
+			gchar *tmp, *source;
+			
+			source = g_strdup (subscription_get_source (subscription));
+			tmp = strrchr (source, '/');
+			if (tmp)
+				*(tmp+1) = '\0';
+
+			htmlUrl = common_build_url (newHtmlUrl, source);
+			g_free (source);
+		}
+		
+		metadata_list_set (&subscription->metadata, "homepage", htmlUrl);
+		g_free (htmlUrl);
+	}
+}
+
 void
 subscription_set_filter (subscriptionPtr subscription, const gchar *filter)
 {
@@ -396,63 +434,11 @@ subscription_set_filter (subscriptionPtr subscription, const gchar *filter)
 	feedlist_schedule_save ();
 }
 
-/**
- * Creates a new error description according to the passed
- * HTTP status and the feeds parser errors. If the HTTP
- * status is a success status and no parser errors occurred
- * no error messages is created.
- *
- * @param feed		feed
- * @param httpstatus	HTTP status
- * @param resultcode	the update code's return code (see update.h)
- */
-void
-subscription_update_error_status (subscriptionPtr subscription,
-                                  gint httpstatus,
-                                  gint resultcode,
-                                  gchar *filterError)
-{
-	const gchar	*errmsg = NULL;
-	gboolean	errorFound = FALSE;
-
-	if (subscription->filterError)
-		g_free (subscription->filterError);
-	if (subscription->httpError)
-		g_free (subscription->httpError);
-	if (subscription->updateError)
-		g_free (subscription->updateError);
-		
-	subscription->filterError = g_strdup (filterError);
-	subscription->updateError = NULL;
-	subscription->httpError = NULL;
-	subscription->httpErrorCode = httpstatus;
-	
-	if (((httpstatus >= 200) && (httpstatus < 400)) && /* HTTP codes starting with 2 and 3 mean no error */
-	    (NULL == subscription->filterError))
-		return;
-	
-	if ((200 != httpstatus) || (resultcode != NET_ERR_OK)) {	
-		/* first specific codes (guarantees tmp to be set) */
-		errmsg = common_http_error_to_str (httpstatus);
-
-		/* second netio errors */
-		if (common_netio_error_to_str (resultcode))
-			errmsg = common_netio_error_to_str (resultcode);
-
-		errorFound = TRUE;
-		subscription->httpError = g_strdup (errmsg);
-	}
-	
-	/* if none of the above error descriptions matched... */
-	if (!errorFound)
-		subscription->updateError = g_strdup (_("There was a problem while reading this subscription. Please check the URL and console output."));
-}
-
 subscriptionPtr
 subscription_import (xmlNodePtr xml, gboolean trusted)
 {
 	subscriptionPtr	subscription;
-	xmlChar		*source, *filter, *intervalStr, *tmp;
+	xmlChar		*source, *homepage, *filter, *intervalStr, *tmp;
 
 	subscription = subscription_new (NULL, NULL, NULL);
 	
@@ -471,7 +457,12 @@ subscription_import (xmlNodePtr xml, gboolean trusted)
 	
 		subscription_set_source (subscription, source);
 		xmlFree (source);
-	
+
+		homepage = xmlGetProp (xml, BAD_CAST "htmlUrl");
+		if (homepage && xmlStrcmp (homepage, ""))
+			subscription_set_homepage (subscription, homepage);
+		xmlFree (homepage);
+
 		if ((filter = xmlGetProp (xml, BAD_CAST "filtercmd"))) {
 			if (!trusted) {
 				/* FIXME: Display warning dialog asking if the command
@@ -494,7 +485,7 @@ subscription_import (xmlNodePtr xml, gboolean trusted)
 		if (tmp && !xmlStrcmp (tmp, BAD_CAST "true"))
 			subscription->updateOptions->dontUseProxy = TRUE;
 		xmlFree (tmp);
-		
+	
 		/* authentication options */
 		subscription->updateOptions->username = xmlGetProp (xml, BAD_CAST "username");
 		subscription->updateOptions->password = xmlGetProp (xml, BAD_CAST "password");
@@ -509,6 +500,11 @@ subscription_export (subscriptionPtr subscription, xmlNodePtr xml, gboolean trus
 	gchar *interval = g_strdup_printf ("%d", subscription_get_update_interval (subscription));
 
 	xmlNewProp (xml, BAD_CAST "xmlUrl", BAD_CAST subscription_get_source (subscription));
+
+	if (subscription_get_homepage (subscription))
+		xmlNewProp (xml, BAD_CAST"htmlUrl", BAD_CAST subscription_get_homepage (subscription));
+	else
+		xmlNewProp (xml, BAD_CAST"htmlUrl", BAD_CAST "");
 	
 	if (subscription_get_filter (subscription))
 		xmlNewProp (xml, BAD_CAST"filtercmd", BAD_CAST subscription_get_filter (subscription));
@@ -528,6 +524,37 @@ subscription_export (subscriptionPtr subscription, xmlNodePtr xml, gboolean trus
 	g_free (interval);
 }
 
+void
+subscription_to_xml (subscriptionPtr subscription, xmlNodePtr xml)
+{
+	gchar	*tmp;
+	
+	xmlNewTextChild (xml, NULL, "feedSource", subscription_get_source (subscription));
+	xmlNewTextChild (xml, NULL, "feedOrigSource", subscription_get_orig_source (subscription));
+
+	tmp = g_strdup_printf ("%d", subscription_get_default_update_interval (subscription));
+	xmlNewTextChild (xml, NULL, "feedUpdateInterval", tmp);
+	g_free (tmp);
+
+	tmp = g_strdup_printf ("%d", subscription->discontinued?1:0);
+	xmlNewTextChild (xml, NULL, "feedDiscontinued", tmp);
+	g_free (tmp);
+
+	if (subscription->updateError)
+		xmlNewTextChild (xml, NULL, "updateError", subscription->updateError);
+	if (subscription->httpError) {
+		xmlNewTextChild (xml, NULL, "httpError", subscription->httpError);
+
+		tmp = g_strdup_printf ("%d", subscription->httpErrorCode);
+		xmlNewTextChild (xml, NULL, "httpErrorCode", tmp);
+		g_free (tmp);
+	}
+	if (subscription->filterError)
+		xmlNewTextChild (xml, NULL, "filterError", subscription->filterError);
+
+	metadata_add_xml_nodes (subscription->metadata, xml);
+}
+
 void
 subscription_free (subscriptionPtr subscription)
 {
