@@ -1,7 +1,7 @@
 /**
  * @file ui_tray.c tray icon handling
  * 
- * Copyright (C) 2003-2007 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2009 Lars Lindner <lars.lindner@gmail.com>
  * Copyright (C) 2004 Christophe Barbe <christophe.barbe@ufies.org>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -32,9 +32,9 @@
 #include "conf.h"
 #include "eggtrayicon.h"
 #include "feedlist.h"
-#include "update.h"
-#include "ui/ui_dnd.h"
-#include "ui/ui_mainwindow.h"
+#include "net.h"
+#include "ui/liferea_shell.h"
+#include "ui/ui_common.h"
 #include "ui/ui_popup.h"
 #include "ui/ui_tray.h"
 
@@ -45,8 +45,6 @@
 #define TRAY_ICON_WIDTH	16
 #define TRAY_ICON_HEIGHT 16
 
-extern GdkPixbuf	*icons[];
-
 static struct trayIcon_priv {
 	int		trayCount;		/**< reference counter */
 	GdkPixbuf	*currentIcon;		/**< currently displayed icon */
@@ -55,30 +53,14 @@ static struct trayIcon_priv {
 	GtkWidget	*image;			/**< render widget in the notification area */
 	GtkWidget	*alignment;
 	GtkWidget	*eventBox;
-	GtkTooltips	*toolTips;
 } *trayIcon_priv = NULL;
 
 static void ui_tray_install(void);
 
-void ui_tray_tooltip_set(gchar *message) {
-	GtkTooltipsData	*data = NULL;
-	
-	g_assert(trayIcon_priv->toolTips);
-
-	data = gtk_tooltips_data_get(GTK_WIDGET(trayIcon_priv->widget));
-
-	if(NULL != data) {
-		/* FIXME: Is this necessary? Gtk 2.2.4 frees these strings
-		   inside of set_tip, if needed. */
-		g_free(data->tip_text);
-		g_free(data->tip_private);
-		data->tip_text = NULL;
-		data->tip_private = NULL;
-	}
-	
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(trayIcon_priv->toolTips), 
-	                     GTK_WIDGET(trayIcon_priv->eventBox),
-			     message, message);
+static void
+ui_tray_tooltip_set (const gchar *message)
+{
+	gtk_widget_set_tooltip_text(GTK_WIDGET(trayIcon_priv->eventBox), message);
 }
 
 static void ui_tray_expose_cb() {
@@ -186,57 +168,57 @@ ui_tray_icon_set (gint newItems, GdkPixbuf *icon)
    	}
 }
 
-void ui_tray_update(void) {
+void
+ui_tray_update (void)
+{
 	gint	newItems, unreadItems;
 	gchar	*msg, *tmp;
 	
-	if(!trayIcon_priv)
+	if (!trayIcon_priv)
 		return;
 
-	newItems = feedlist_get_new_item_count();
-	unreadItems = feedlist_get_unread_item_count();
+	newItems = feedlist_get_new_item_count ();
+	unreadItems = feedlist_get_unread_item_count ();
 		
-	if(newItems != 0) {
-		if(update_is_online())
-			ui_tray_icon_set(newItems, icons[ICON_AVAILABLE]);
+	if (newItems != 0) {
+		if (network_is_online ())
+			ui_tray_icon_set (newItems, icons[ICON_AVAILABLE]);
 		else
-			ui_tray_icon_set(newItems, icons[ICON_AVAILABLE_OFFLINE]);
+			ui_tray_icon_set (newItems, icons[ICON_AVAILABLE_OFFLINE]);
 			
-		msg = g_strdup_printf(ngettext("%d new item", "%d new items", newItems), newItems);
+		msg = g_strdup_printf (ngettext ("%d new item", "%d new items", newItems), newItems);
 	} else {
-		if(update_is_online())
-			ui_tray_icon_set(newItems, icons[ICON_EMPTY]);
+		if (network_is_online ())
+			ui_tray_icon_set (newItems, icons[ICON_EMPTY]);
 		else
-			ui_tray_icon_set(newItems, icons[ICON_EMPTY_OFFLINE]);
+			ui_tray_icon_set (newItems, icons[ICON_EMPTY_OFFLINE]);
 			
-		msg = g_strdup(_("No new items"));
+		msg = g_strdup (_("No new items"));
 	}
 
-	if(unreadItems != 0) {
-		tmp = g_strdup_printf(ngettext("%s\n%d unread item", "%s\n%d unread items", unreadItems), msg, unreadItems);
-	} else {
-		tmp = g_strdup_printf(_("%s\nNo unread items"), msg);
-	}
+	if (unreadItems != 0)
+		tmp = g_strdup_printf (ngettext("%s\n%d unread item", "%s\n%d unread items", unreadItems), msg, unreadItems);
+	else
+		tmp = g_strdup_printf (_("%s\nNo unread items"), msg);
 
-	ui_tray_tooltip_set(tmp);
-	g_free(tmp);
-	g_free(msg);
+	ui_tray_tooltip_set (tmp);
+	g_free (tmp);
+	g_free (msg);
 }
 
 /* a click on the systray icon should show the program window
    if invisible or hide it if visible */
-static void tray_icon_pressed(GtkWidget *button, GdkEventButton *event, EggTrayIcon *icon) {
-	
-	switch(event->button) {
+static void
+tray_icon_pressed (GtkWidget *button, GdkEventButton *event, EggTrayIcon *icon)
+{	
+	switch (event->button) {
 		case 1:
-			ui_mainwindow_toggle_visibility(NULL, NULL);
+			liferea_shell_toggle_visibility ();
 			break;
 		case 3:
-			gtk_menu_popup(ui_popup_make_systray_menu(), NULL, NULL, NULL, NULL, event->button, event->time);
+			ui_popup_systray_menu (event->button, event->time);
 			break;
 	}
-						  
-	return;
 }
 
 static gboolean ui_tray_create_cb() {
@@ -247,9 +229,10 @@ static gboolean ui_tray_create_cb() {
 }
 
 
-static void ui_tray_embedded_cb(GtkWidget *widget, void *data) {
-
-	ui_mainwindow_tray_add();
+static void
+ui_tray_embedded_cb(GtkWidget *widget, void *data)
+{
+	/* Nothing to do */
 }
 
 
@@ -259,7 +242,8 @@ static void ui_tray_destroyed_cb(GtkWidget *widget, void *data) {
 	g_free(trayIcon_priv);
 	trayIcon_priv = NULL;
 	
-	ui_mainwindow_tray_remove();
+	if (0 == ui_tray_get_count ())
+		liferea_shell_present ();
 	
 	/* And make it re-appear when the notification area reappears */
 	g_idle_add(ui_tray_create_cb, NULL);
@@ -281,12 +265,11 @@ static void ui_tray_install(void) {
 	g_signal_connect(G_OBJECT(trayIcon_priv->widget), "destroy",
 	                 G_CALLBACK(ui_tray_destroyed_cb), NULL);
 	
-	ui_dnd_setup_URL_receiver(trayIcon_priv->eventBox);
+	liferea_shell_setup_URL_receiver (trayIcon_priv->eventBox);
 	
 	gtk_container_add(GTK_CONTAINER(trayIcon_priv->widget), trayIcon_priv->eventBox);
 	g_object_ref(G_OBJECT(trayIcon_priv->widget));
 	
-	trayIcon_priv->toolTips = gtk_tooltips_new();
 	ui_tray_update();
 	trayIcon_priv->trayCount++;
 }
@@ -303,7 +286,8 @@ static void ui_tray_remove(void) {
 	g_free(trayIcon_priv);
 	trayIcon_priv = NULL;
 
-	ui_mainwindow_tray_remove();
+	if (0 == ui_tray_get_count ())
+		liferea_shell_present ();
 }
 
 void ui_tray_enable(gboolean enabled) {
@@ -317,7 +301,9 @@ void ui_tray_enable(gboolean enabled) {
 	}
 }
 
-int ui_tray_get_count() {
+guint
+ui_tray_get_count(void)
+{
 	return trayIcon_priv?trayIcon_priv->trayCount:0;
 }
 
