@@ -1,8 +1,9 @@
 /**
- * @file update.c  generic update request processing
+ * @file update.c  generic update request and state processing
  *
- * Copyright (C) 2003-2007 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2008 Lars Lindner <lars.lindner@gmail.com>
  * Copyright (C) 2004-2006 Nathan J. Conrad <t98502@users.sourceforge.net>
+ * Copyright (C) 2009 Adrian Bunk <bunk@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +20,8 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include "update.h"
+
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -35,6 +38,7 @@
 #include <string.h>
 
 #ifdef USE_NM
+#include <dbus/dbus.h>
 #include <libnm_glib.h>
 #endif
 
@@ -42,32 +46,21 @@
 #include "conf.h"
 #include "debug.h"
 #include "net.h"
-#include "update.h"
 #include "xml.h"
-#include "ui/ui_htmlview.h"
-#include "ui/ui_mainwindow.h"
+#include "ui/liferea_htmlview.h"
+#include "ui/liferea_shell.h"
 #include "ui/ui_tray.h"
 
-/* must never be smaller than 2, because first thread works exclusivly on high prio requests */
-#define DEFAULT_UPDATE_THREAD_CONCURRENCY	6
-
 /** global update job list, used for lookups when cancelling */
-static GSList *jobs = NULL;
+static GSList	*jobs = NULL;
 
-/** list of update threads */
-static GSList *threads = NULL;
-
-/* communication queues for requesting updates and sending the results */
-static GAsyncQueue	*pendingHighPrioJobs = NULL;
-static GAsyncQueue	*pendingJobs = NULL;
-static GAsyncQueue	*finishedJobs = NULL;
-
-static guint		results_timer = 0;
-
-/* condition mutex for offline mode */
-static GMutex	*cond_mutex = NULL;
-static GCond	*offline_cond = NULL;
-static gboolean	online = TRUE;
+/* We use a communication queue for limiting the number 
+   of concurrent requests to avoid hitting the file descriptor
+   limit of the glibcurl code. */
+static GAsyncQueue *pendingHighPrioJobs = NULL;
+static GAsyncQueue *pendingJobs = NULL;
+static guint numberOfActiveJobs = 0;
+#define MAX_ACTIVE_JOBS	5
 
 #ifdef USE_NM
 /* State for NM support */
@@ -75,8 +68,6 @@ static libnm_glib_ctx *nm_ctx = NULL;
 static guint nm_id = 0;
 #endif
 
-static gboolean update_dequeue_finished_jobs (gpointer user_data);
-
 /* update state interface */
 
 updateStatePtr
@@ -85,34 +76,16 @@ update_state_new (void)
 	return g_new0 (struct updateState, 1);
 }
 
-const gchar *
+glong
 update_state_get_lastmodified (updateStatePtr state)
 {
 	return state->lastModified;
 }
 
 void
-update_state_set_lastmodified (updateStatePtr state, const gchar *lastModified)
+update_state_set_lastmodified (updateStatePtr state, glong lastModified)
 {
-	g_free (state->lastModified);
-	state->lastModified = NULL;
-	if (lastModified)
-		state->lastModified = g_strdup (lastModified);
-}
-
-const gchar *
-update_state_get_etag (updateStatePtr state)
-{
-	return state->etag;
-}
-
-void
-update_state_set_etag (updateStatePtr state, const gchar *etag)
-{
-	g_free (state->etag);
-	state->etag = NULL;
-	if (etag)
-		state->etag = g_strdup (etag);
+	state->lastModified = lastModified;
 }
 
 const gchar *
@@ -136,7 +109,6 @@ update_state_copy (updateStatePtr state)
 	updateStatePtr newState;
 	
 	newState = update_state_new ();
-	update_state_set_etag (newState, update_state_get_etag (state));
 	update_state_set_lastmodified (newState, update_state_get_lastmodified (state));
 	update_state_set_cookies (newState, update_state_get_cookies (state));
 	
@@ -144,15 +116,69 @@ update_state_copy (updateStatePtr state)
 }
 
 void
-update_state_free (updateStatePtr state)
+update_state_free (updateStatePtr updateState)
 {
-	if (!state)
+	if (!updateState)
 		return;
 
-	g_free (state->etag);
-	g_free (state->lastModified);
-	g_free (state->cookies);
-	g_free (state);
+	g_free (updateState->cookies);
+	g_free (updateState);
+}
+
+/* update request processing */
+
+updateRequestPtr
+update_request_new (void)
+{
+	return g_new0 (struct updateRequest, 1);
+}
+
+void
+update_request_free (updateRequestPtr request)
+{
+	if (!request)
+		return;
+	
+	update_state_free (request->updateState);
+	update_options_free (request->options);
+
+	g_free (request->postdata);
+	g_free (request->source);
+	g_free (request->filtercmd);
+	g_free (request);
+}
+
+void
+update_request_set_source(updateRequestPtr request, gchar* source) 
+{
+	g_free (request->source);
+	request->source = g_strdup(source) ;
+}
+
+updateResultPtr
+update_result_new (void)
+{
+	updateResultPtr	result;
+	
+	result = g_new0 (struct updateResult, 1);
+	result->updateState = update_state_new ();
+	
+	return result;
+}
+
+void
+update_result_free (updateResultPtr result)
+{
+	if (!result)
+		return;
+		
+	update_state_free (result->updateState);
+
+	g_free (result->data);
+	g_free (result->source);
+	g_free (result->contentType);
+	g_free (result->filterErrors);
+	g_free (result);
 }
 
 updateOptionsPtr
@@ -179,66 +205,8 @@ update_options_free (updateOptionsPtr options)
 	g_free (options);
 }
 
-/* update request processing */
-
-updateRequestPtr
-update_request_new (void)
-{
-	return g_new0 (struct updateRequest, 1);
-}
-
-static void
-update_request_free (updateRequestPtr request)
-{
-	if (!request)
-		return;
-	
-	update_state_free (request->updateState);
-	update_options_free (request->options);
-	
-	g_free (request->source);
-	g_free (request->filtercmd);
-	g_free (request);
-}
-
-updateResultPtr
-update_result_new (void)
-{
-	updateResultPtr	result;
-	
-	result = g_new0 (struct updateResult, 1);
-	result->updateState = update_state_new ();
-	
-	return result;
-}
-
-void
-update_result_free (updateResultPtr result)
-{
-	if (!result)
-		return;
-		
-	update_state_free (result->updateState);
-
-	g_free (result->data);
-	g_free (result->contentType);
-	g_free (result->filterErrors);
-	g_free (result);
-}
-
 /* update job handling */
 
-typedef struct updateJob {
-	updateRequestPtr	request;
-	updateResultPtr		result;
-	gpointer		owner;		/**< owner of this job (used for matching when cancelling) */
-	update_result_cb	callback;	/**< result processing callback */
-	gpointer		user_data;	/**< result processing user data */
-	updateFlags		flags;		/**< request and result processing flags */
-	gint			state;		/**< State of the job (enum request_state) */	
-	gushort			retriesCount;	/**< Count how many retries have been done */	
-} *updateJobPtr;
-
 static updateJobPtr
 update_job_new (gpointer owner,
                 updateRequestPtr request,
@@ -251,6 +219,7 @@ update_job_new (gpointer owner,
 	job = g_new0 (struct updateJob, 1);
 	job->owner = owner;
 	job->request = request;
+	job->result = update_result_new ();
 	job->callback = callback;
 	job->user_data = user_data;
 	job->flags = flags;	
@@ -290,7 +259,7 @@ update_exec_filter_cmd (gchar *cmd, gchar *data, gchar **errorOutput, size_t *si
 	FILE		*file, *p;
 	
 	*errorOutput = NULL;
-	tmpfilename = g_strdup_printf("%s" G_DIR_SEPARATOR_S "liferea-XXXXXX", tmpdir);
+	tmpfilename = g_build_filename (tmpdir, "liferea-XXXXXX", NULL);
 	
 	fd = g_mkstemp(tmpfilename);
 	
@@ -390,7 +359,7 @@ static void
 update_apply_filter (updateJobPtr job)
 {
 	gchar	*filterResult;
-	size_t	len;
+	size_t	len = 0;
 
 	g_assert (NULL == job->result->filterErrors);
 
@@ -438,9 +407,11 @@ update_exec_cmd (updateJobPtr job)
 		if (job->result->data)
 			job->result->data[job->result->size] = '\0';
 	} else {
-		ui_mainwindow_set_status_bar (_("Error: Could not open pipe \"%s\""), (job->request->source) + 1);
+		liferea_shell_set_status_bar (_("Error: Could not open pipe \"%s\""), (job->request->source) + 1);
 		job->result->httpstatus = 404;	/* FIXME: maybe setting request->returncode would be better */
 	}
+	
+	update_process_finished_job (job);
 }
 
 static void
@@ -462,109 +433,79 @@ update_load_file (updateJobPtr job)
 		/* we have a file... */
 		if ((!g_file_get_contents (filename, &(job->result->data), &(job->result->size), NULL)) || (job->result->data[0] == '\0')) {
 			job->result->httpstatus = 403;	/* FIXME: maybe setting request->returncode would be better */
-			ui_mainwindow_set_status_bar (_("Error: Could not open file \"%s\""), filename);
+			liferea_shell_set_status_bar (_("Error: Could not open file \"%s\""), filename);
 		} else {
 			job->result->httpstatus = 200;
+			debug2 (DEBUG_UPDATE, "Successfully read %d bytes from file %s.", job->result->size, filename);
 		}
 	} else {
-		ui_mainwindow_set_status_bar (_("Error: There is no file \"%s\""), filename);
+		liferea_shell_set_status_bar (_("Error: There is no file \"%s\""), filename);
 		job->result->httpstatus = 404;	/* FIXME: maybe setting request->returncode would be better */
 	}
+	
+	update_process_finished_job (job);
 }
 
 static void
-update_job_run (updateJobPtr job) 
+update_job_run (updateJobPtr job)
 {
-	if (*(job->request->source) == '|') {
-		update_exec_cmd (job);
-		
-	} else if (strstr (job->request->source, "://") && 
-	           strncmp (job->request->source, "file://", 7)) {
-		job->result = network_process_request (job->request);
-		if (job->result->httpstatus >= 400) {
-			g_free (job->result->data);
-			job->result->data = NULL;
-			job->result->size = 0;
-		}
-	} else {
-		update_load_file (job);		
-	}
-
-	/* Finally execute the postfilter */
-	if (job->result->data && job->request->filtercmd) 
-		update_apply_filter (job);
-		
-}
-
-updateResultPtr
-update_execute_request_sync (gpointer owner, 
-                             updateRequestPtr request, 
-			     guint flags)
-{
-	updateResultPtr	result;
-	updateJobPtr	job;
+	/* Here we decide on the source type and the proper execution
+	   methods which then do anything they want with the job and
+	   pass the processed job to update_process_finished_job()
+	   for result dequeuing */
 	
-	job = update_job_new (owner, request, NULL, NULL, flags);
-	update_job_run (job);
-	result = job->result;
-	job->result = NULL;
-	update_job_free (job);
-			
-	return result;
+	/* everything starting with '|' is a local command */
+	if (*(job->request->source) == '|') {
+		debug1 (DEBUG_UPDATE, "Recognized local command: %s", job->request->source);
+		update_exec_cmd (job);
+		return;
+	}
+	
+	/* if it has a protocol "://" prefix, but not "file://" it is an URI */
+	if (strstr (job->request->source, "://") && strncmp (job->request->source, "file://", 7)) {
+		network_process_request (job);
+		return;
+	}
+	
+	/* otherwise it must be a local file... */
+	{
+		debug1 (DEBUG_UPDATE, "Recognized file URI: %s", job->request->source);
+		update_load_file (job);
+		return;
+	}
 }
 
-static void *
-update_dequeue_jobs (void *data)
+static gboolean
+update_dequeue_job (gpointer user_data)
 {
-	updateJobPtr	job;
-	gboolean	high_priority = (gboolean)GPOINTER_TO_INT (data);
-
-	for (;;) {
-		/* block updating if we are offline */
-		if (!online) {
-			debug0 (DEBUG_UPDATE, "now going offline!");
-			g_mutex_lock (cond_mutex);
-			g_cond_wait (offline_cond, cond_mutex);
-	                g_mutex_unlock (cond_mutex);
-			debug0 (DEBUG_UPDATE, "going online again!");
-		}
+	updateJobPtr job;
+	
+	if (!pendingJobs)
+		return FALSE;	/* we must be in shutdown */
 		
-		/* do update processing */
-		if (DEBUG_VERBOSE & debug_level)
-			debug0 (DEBUG_UPDATE, "waiting for request...");
-		if (high_priority) {
-			job = g_async_queue_pop (pendingHighPrioJobs);
-		} else {
-			do {
-				job = g_async_queue_try_pop (pendingHighPrioJobs);
-				if (!job) {
-					GTimeVal wait;
-					g_get_current_time (&wait);
-					g_time_val_add (&wait, 500000);
-					job = g_async_queue_timed_pop (pendingJobs, &wait);
-				}
-			} while (!job);
-		}
-		g_assert (NULL != job);
-		job->state = REQUEST_STATE_PROCESSING;
+	if (numberOfActiveJobs >= MAX_ACTIVE_JOBS)
+		return TRUE;	/* let's continue later */
 
-		debug1 (DEBUG_UPDATE, "processing request (%s)", job->request->source);
-		if (job->callback == NULL) {
-			debug1 (DEBUG_UPDATE, "freeing cancelled request (%s)", job->request->source);
-			update_job_free (job);
-		} else {
-			update_job_run (job);
-			
-			/* return the request so the GUI thread can merge the feeds and display the results... */
-			debug1 (DEBUG_UPDATE, "request (%s) finished", job->request->source);
-			g_async_queue_push (finishedJobs, (gpointer)job);
-			if (!results_timer) 
-				results_timer = g_timeout_add_full (G_PRIORITY_DEFAULT_IDLE, 100,
-				                                    update_dequeue_finished_jobs, 
-								    NULL, NULL);
+	job = (updateJobPtr)g_async_queue_try_pop(pendingHighPrioJobs);
 
-		}
+	if (!job)
+		job = (updateJobPtr)g_async_queue_try_pop(pendingJobs);
+
+	if(!job)
+		return TRUE;	/* no request at the moment */
+
+	numberOfActiveJobs++;
+
+	job->state = REQUEST_STATE_PROCESSING;
+
+	debug1 (DEBUG_UPDATE, "processing request (%s)", job->request->source);
+	if (job->callback == NULL) {
+		update_process_finished_job (job);
+	} else {
+		update_job_run (job);
 	}
+		
+	return TRUE;
 }
 
 updateJobPtr
@@ -579,15 +520,16 @@ update_execute_request (gpointer owner,
 	g_assert (request->options != NULL);
 	
 	job = update_job_new (owner, request, callback, user_data, flags);
-	job->state = REQUEST_STATE_PENDING;
-	
+	job->state = REQUEST_STATE_PENDING;	
 	jobs = g_slist_append (jobs, job);
-	
-	if (flags & FEED_REQ_PRIORITY_HIGH)
-		g_async_queue_push (pendingHighPrioJobs, job);
-	else
-		g_async_queue_push (pendingJobs, job);
-		
+
+	if (flags & FEED_REQ_PRIORITY_HIGH) {
+		g_async_queue_push (pendingHighPrioJobs, (gpointer)job);
+		update_dequeue_job (NULL);
+	} else {
+		g_async_queue_push (pendingJobs, (gpointer)job);
+	}
+
 	return job;
 }
 
@@ -604,83 +546,6 @@ update_job_cancel_by_owner (gpointer owner)
 	}
 }
 
-void
-update_set_online (gboolean mode)
-{
-	if (online != mode) {
-		online = mode;
-		if (online) {
-			g_mutex_lock (cond_mutex);
-			g_cond_signal (offline_cond);
-			g_mutex_unlock (cond_mutex);
-		}
-		debug1 (DEBUG_UPDATE, "Changing online mode to %s", mode?"online":"offline");
-		ui_mainwindow_online_status_changed (mode);
-		liferea_htmlview_set_online (mode);
-		ui_tray_update ();
-	}
-}
-
-gboolean
-update_is_online (void)
-{
-	return online;
-}
-
-/* Wrapper for reenqueuing requests in case of retries, for convenient call from g_timeout */
-static gboolean
-update_requeue_job (gpointer data)
-{
-	updateJobPtr job = (updateJobPtr)data;
-	
-	if (job->callback == NULL) {
-		debug2(DEBUG_UPDATE, "Freeing request of cancelled retry #%d for \"%s\"", job->retriesCount, job->request->source);
-		update_job_free (job);
-	} else {
-		g_async_queue_push (pendingJobs, job);
-	}
-	return FALSE;
-}
-
-/* Schedules a retry for the given request */
-static void
-update_job_retry (updateJobPtr job)
-{
-	guint retryDelay;
-	gushort i;	
-
-	if (job->result) {
-		update_result_free (job->result);
-		job->result = NULL;
-	}
-		
-	/* Note: in case of permanent HTTP redirection leading to a network
-	 * error, retries will be done on the redirected request->source. */
-
-	/* Prepare for a retry: increase counter and calculate delay */
-	retryDelay = REQ_MIN_DELAY_FOR_RETRY;
-	for (i = 0; i < job->retriesCount; i++)
-		retryDelay *= 3;
-	if (retryDelay > REQ_MAX_DELAY_FOR_RETRY)
-		retryDelay = REQ_MAX_DELAY_FOR_RETRY;
-
-	/* Requeue the request after the waiting delay */
-	job->retriesCount++;	
-	g_timeout_add_full (G_PRIORITY_DEFAULT_IDLE, 1000 * retryDelay, update_requeue_job, job, NULL);
-	ui_mainwindow_set_status_bar (_("Could not download \"%s\". Will retry in %d seconds."), job->request->source, retryDelay);
-}
-
-gboolean
-update_job_cancel_retry (updateJobPtr job)
-{
-	if (0 < job->retriesCount) {
-		job->callback = NULL;
-		debug2 (DEBUG_UPDATE, "cancelling retry #%d (%s)", job->retriesCount, job->request->source);
-		return TRUE;
-	}
-	return FALSE;
-}
-
 static gboolean
 update_process_result_idle_cb (gpointer user_data)
 {
@@ -694,66 +559,26 @@ update_process_result_idle_cb (gpointer user_data)
 	return FALSE;
 }
 
-static gboolean
-update_dequeue_finished_jobs (gpointer user_data)
-{
-	updateJobPtr	job;
-
-	results_timer = 0;
-	
-	while (NULL != (job = g_async_queue_try_pop (finishedJobs))) {
-		job->state = REQUEST_STATE_DEQUEUE;
-
-		/* Handling abandoned requests (e.g. after feed deletion) */
-		if (job->callback == NULL) {	
-			debug1 (DEBUG_UPDATE, "freeing cancelled request (%s)", job->request->source);
-			update_job_free (job);
-			continue;
-		} 
-		
-		/* Retrying in some error cases */
-		if ((job->result->returncode == NET_ERR_UNKNOWN) ||
-		    (job->result->returncode == NET_ERR_CONN_FAILED) ||
-		    (job->result->returncode == NET_ERR_SOCK_ERR) ||
-		    (job->result->returncode == NET_ERR_HOST_NOT_FOUND) ||
-		    (job->result->returncode == NET_ERR_TIMEOUT)) {
-
-			if (job->request->allowRetries && (REQ_MAX_NUMBER_OF_RETRIES <= job->retriesCount)) {
-				debug1 (DEBUG_UPDATE, "retrying download (%s)", job->request->source);
-				update_job_retry (job);
-				continue;
-			}
-
-			debug1 (DEBUG_UPDATE, "retry count exceeded (%s)", job->request->source);
-		}
-		
-		/* Normal result processing */
-		g_idle_add (update_process_result_idle_cb, job);
-	}
-	return FALSE;
-}
-
 void
-update_init (void)
+update_process_finished_job (updateJobPtr job)
 {
-	gushort	i, count;
+	job->state = REQUEST_STATE_DEQUEUE;
+	
+	g_assert(numberOfActiveJobs > 0);
+	numberOfActiveJobs--;
 
-	network_init ();
-	
-	pendingHighPrioJobs = g_async_queue_new ();
-	pendingJobs = g_async_queue_new ();
-	finishedJobs = g_async_queue_new ();
-	
-	offline_cond = g_cond_new ();
-	cond_mutex = g_mutex_new ();
+	/* Handling abandoned requests (e.g. after feed deletion) */
+	if (job->callback == NULL) {	
+		debug1 (DEBUG_UPDATE, "freeing cancelled request (%s)", job->request->source);
+		update_job_free (job);
+		return;
+	} 
+
+	/* Finally execute the postfilter */
+	if (job->result->data && job->request->filtercmd) 
+		update_apply_filter (job);
 		
-	if (1 >= (count = conf_get_int_value (UPDATE_THREAD_CONCURRENCY)))
-		count = DEFAULT_UPDATE_THREAD_CONCURRENCY;
-	
-	for (i = 0; i < count; i++) {
-		GThread *thread = g_thread_create (update_dequeue_jobs, GINT_TO_POINTER((i == 0)), FALSE, NULL);
-		threads = g_slist_append (threads, thread);
-	}
+	g_idle_add (update_process_result_idle_cb, job);
 }
 
 #ifdef USE_NM
@@ -766,18 +591,18 @@ update_network_monitor (libnm_glib_ctx *ctx, gpointer user_data)
 	g_return_if_fail (ctx != NULL);
 
 	state = libnm_glib_get_network_state (ctx);
-	online = update_is_online ();
+	online = network_is_online ();
 
 	if (online && state == LIBNM_NO_NETWORK_CONNECTION) {
 		debug0 (DEBUG_UPDATE, "network manager: no network connection -> going offline");
-		update_set_online (FALSE);
+		network_set_online (FALSE);
 	} else if (!online && state == LIBNM_ACTIVE_NETWORK_CONNECTION) {
 		debug0 (DEBUG_UPDATE, "network manager: active connection -> going online");
-		update_set_online (TRUE);
+		network_set_online (TRUE);
 	}
 }
 
-gboolean
+static gboolean
 update_nm_initialize (void)
 {
 	debug0 (DEBUG_UPDATE, "network manager: registering network state change callback");
@@ -795,7 +620,7 @@ update_nm_initialize (void)
 	return TRUE;
 }
 
-void
+static void
 update_nm_cleanup (void)
 {
 	debug0 (DEBUG_UPDATE, "network manager: unregistering network state change callback");
@@ -809,28 +634,64 @@ update_nm_cleanup (void)
 }
 #endif
 
+void
+update_init (void)
+{
+	pendingJobs = g_async_queue_new ();
+	pendingHighPrioJobs = g_async_queue_new ();
+	network_init ();
+	
+	g_timeout_add (500, update_dequeue_job, NULL);
+	
+#ifdef USE_NM
+	{
+		DBusConnection *connection;
+
+		connection = dbus_bus_get (DBUS_BUS_SYSTEM, NULL);
+
+		if (connection) {
+			dbus_connection_set_exit_on_disconnect (connection, FALSE);
+
+			if (dbus_bus_name_has_owner (connection, "org.freedesktop.NetworkManager", NULL)) {
+				update_nm_initialize ();
+				/* network manager will set online state right after initialization... */
+			} else {
+				network_set_online (TRUE);
+			}
+
+			dbus_connection_unref(connection);
+
+			return;
+		}
+	}
+#endif
+	network_set_online (TRUE);
+}
+
 void
 update_deinit (void)
 {
+	GSList	*iter = jobs;
+
 	debug_enter ("update_deinit");
+
+#ifdef USE_NM	
+	update_nm_cleanup ();
+#endif	
+	network_deinit ();
 	
-	//update_nm_cleanup ();
-	
-	//network_deinit ();
-	
-	/* FIXME: terminate update threads to be able to remove the queues
-	
-	g_async_queue_unref (pendingHighPrioJobs);
+	/* Cancel all jobs, to avoid async callbacks accessing the GUI */
+	while (iter) {
+		updateJobPtr job = (updateJobPtr)iter->data;
+		job->callback = NULL;
+		iter = g_slist_next (iter);
+	}
+
 	g_async_queue_unref (pendingJobs);
-	g_async_queue_unref (finishedJobs);
-	
-	
-	g_free (offline_cond);
-	g_free (cond_mutex);
+	g_async_queue_unref (pendingHighPrioJobs);
 	
 	g_slist_free (jobs);
 	jobs = NULL;
-	*/
 	
 	debug_exit ("update_deinit");
 }
