@@ -1,5 +1,5 @@
 /**
- * @file xml.h XML helper methods for Liferea
+ * @file xml.h  XML helper methods for Liferea
  * 
  * Copyright (C) 2003-2009  Lars Lindner <lars.lindner@gmail.com>
  * Copyright (C) 2004-2006  Nathan J. Conrad <t98502@users.sourceforge.net>
@@ -26,7 +26,7 @@
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
 
-#include "feed.h"
+#include "feed_parser.h"
 
 /**
  * Initialize XML parsing.
@@ -76,15 +76,6 @@ gchar * xhtml_extract (xmlNodePtr cur, gint xhtmlMode, const gchar *defaultBase)
  */
 gchar * xhtml_strip_dhtml (const gchar *html);
 
-/**
- * Strips Flash embedding tags from the given HTML string.
- *
- * @param html	some HTML content
- *
- * @return newly allocated stripped HTML string
- */
-gchar * xhtml_strip_flash (const gchar *html);
-
 /**
  * Convert the given string to proper XHTML content.
  * Note: this function does not respect relative URLs
@@ -111,7 +102,7 @@ gboolean xhtml_is_well_formed (const gchar *text);
  *
  * @return first node found that matches expr (or NULL)
  */
-xmlNodePtr xpath_find (xmlNodePtr node, gchar *expr);
+xmlNodePtr xpath_find (xmlNodePtr node, const gchar *expr);
 
 /** Function type used by common_xpath_foreach_match() */
 typedef void (*xpathMatchFunc)(xmlNodePtr match, gpointer user_data);
@@ -127,6 +118,28 @@ typedef void (*xpathMatchFunc)(xmlNodePtr match, gpointer user_data);
  */
 gboolean xpath_foreach_match (xmlNodePtr node, gchar *expr, xpathMatchFunc func, gpointer user_data);
 
+/**
+ * Return the value of a attribute.
+ *
+ * @param node		XML node
+ * @param name		attribute name
+ *
+ * @returns the attribute value (or NULL) to be free'd with g_free
+ */
+gchar * xml_get_attribute (xmlNodePtr node, const gchar *name);
+
+/**
+ * Return the value of a attribute.
+ * This is the namespace sensitive version of xml_get_attribute().
+ *
+ * @param node		XML node
+ * @param name		attribute name
+ * @param namespace	attribute namespace
+ *
+ * @returns the attribute value (or NULL) to be free'd with g_free
+ */
+gchar * xml_get_ns_attribute (xmlNodePtr node, const gchar *name, const gchar *namespace);
+
 /** used to keep track of error messages during parsing */
 typedef struct errorCtxt {
 	GString		*msg;		/**< message buffer */
